package RSSReader::Exporter::CSV;

use strict;
use RSSReader::Category;
use RSSReader::Feed;
use RSSReader::Exporter;
use Data::Dumper;
use base qw( RSSReader::Logger );


sub new {
  my $c = shift;
  my %a = @_;

  return bless { }, $c
}

sub export {
  my $s = shift;
  my %a = @_;
  my $feeds = $a{feeds};
  my $file_name = $a{'0_filename'};
  my $sep = $a{'1_field_separator'};
  my $line = $a{'2_record_separator'};
  my $header = $a{'3_print_header'};
  my %chars = (
       '\n' => "\n",
       '\t' => "\t",
       '\r\n' => "\r\n",
       '\r' => "\r",
      );
  $line = $chars{$line} if exists $chars{$line};
  $sep  = $chars{$sep} if exists $chars{$sep};

  unless (open (FH, '>',$file_name)) {
    $s->log_warn("Can't open file for write: $!...");
    $@ = "Could not open file for write";
    return undef;
  }
  if ($header) {
    print FH "\"Name\"$sep\"Url\"$line";
  }
  foreach my $feed (@$feeds) {
    my $label = $feed->label;
    $label =~ s/\"/\\\"/g;
    my $url = $feed->url;
    $url =~ s/\"/\\\"/g;
    print FH '"'.$label."\"$sep\"".$url."\"$line";
  }
  close FH;
}

sub id { return "exporter_csv"; }

sub name { return "CSV"; }

sub config_options {
  my $s=shift; 
  return { 
  	   '0_filename' => {
            			default_value   => '', 
            			desc            => 'Filename', 
            			type            => 'file',
	    			mandatory       => 1,
			        },
  	   '1_field_separator' => {
            			default_value   => ';', 
            			desc            => 'Field Separator', 
            			type            => 'text',
	    			mandatory       => 1,
			        },
  	   '2_record_separator' => {
            			default_value   => '\n', 
            			desc            => 'Record Separator', 
            			type            => 'text',
	    			mandatory       => 1,
			        },
  	   '3_print_header' => {
            			default_value   => 0, 
            			desc            => 'Print header row', 
            			type            => 'boolean',
	    			mandatory       => 0,
			        },
         };
}

sub description {
	return 'Export your subscriptions to a Comma Separated Values file';
}

RSSReader::Exporter->register( id => __PACKAGE__->id, package => __PACKAGE__ );
1;
