package RSSReader::Exporter::OPML;

use strict;
use RSSReader::Category;
use RSSReader::Feed;
use RSSReader::Exporter;
use Data::Dumper;
use base qw( RSSReader::Logger );


sub new {
  my $c = shift;
  my %a = @_;

  return bless { }, $c
}

sub export {
  my $s = shift;
  my %a = @_;
  my $feeds = $a{feeds};
  my $file_name = $a{'0_filename'};

  unless (open (FH, '>',$file_name)) {
    $s->log_warn("Can't open file for write: $!...");
    $@ = "Could not open file for write";
    return undef;
  }
  print FH _header();
  foreach my $feed (@$feeds) {
    my $label = $feed->label;
    $label =~ s/\"/\&quote;/g;
    my $url = $feed->url;
    print FH '                <outline title="'.$label.'" type="rss" xmlUrl="'.$url."\"/>\n";
  }
  print FH _footer();
  close FH;
}

sub _header {
  return '<?xml version="1.0" encoding="ISO-8859-1"?>
<!-- OPML generated by Syndigator -->
<opml version="1.1">
        <head>
                <title>Sindigator feeds</title>
                <dateCreated>'.scalar(localtime()).'</dateCreated>
        </head>
        <body>
';
}
sub _footer {
  return '        </body>
</opml>
';
}

sub id { return "exporter_opml"; }

sub name { return "OPML"; }

sub config_options {
  my $s=shift; 
  return { 
  	   '0_filename' => {
            			default_value   => 'export.opml', 
            			desc            => 'Filename', 
            			type            => 'file',
	    			mandatory       => 1,
			        },
         };
}

sub description {
	return 'Export your subscriptions to an OPML file';
}

RSSReader::Exporter->register( id => __PACKAGE__->id, package => __PACKAGE__ );
1;
