package RSSReader::Feed::Message;
use strict;
use RSSReader::Feed;
use RSSReader::Configuration;
use RSSReader::Notifier;
use Data::Dumper;
use base qw( RSSReader::Notifier RSSReader::Logger );

my $_messages;

sub create {
	my $s = shift;
	my %a = @_;

	my %ins = %a;
	delete $ins{feed};

	my $conf = new RSSReader::Configuration;
        my $feed = $a{feed};
        delete $a{feed};
	my $n_msg = $conf->insert_message(
		%a,
		feed_id 	=> $feed->id(),
		title   	=> $a{title},
		date    	=> $a{date},
		url		=> $a{url},
		description 	=> $a{description},
		fetch_time	=> $a{fetch_time},
		comments	=> $a{comments},
		content		=> $a{content},
		author		=> $a{author},
		subject		=> $a{category},
	);

	return undef unless $n_msg;

	my $f = __PACKAGE__->new(id => $n_msg);
        return undef unless $f; 
	$feed->_new_message(message => $f);
	RSSReader->new()->notify('new_message', message=>$f);
	return $f;
}

sub new {
    	#auto connect to feed to warn him of changes.
        my $c = shift;
	my %a = @_;

	if ( exists($_messages->{$a{id}}) ) { return $_messages->{$a{id}} }


       	my $config    = RSSReader::Configuration->new();
       	my $msg_info  = $config->get_message(id=>$a{id});

	if (!$msg_info) {
		$@ = __PACKAGE__."No such message could not fecth info: $@";
		return undef;
	}

       	$c->log_spam("The msg  $a{id} was not cached");
	
       	$_messages->{$a{id}} = bless { id=> $a{id}, info => $msg_info}, $c;

       	return $_messages->{$a{id}};
}

sub updated {
	my $s = shift;
	if ( $s->{info}->{updated} ) {
		return { timestamp => $s->{info}->{last_updated},
			 date      => scalar(localtime($s->{info}->{last_updated})),
			};
	}
	return undef;
}

sub last_text { return shift->{info}->{last_text}; }

sub update {
	my $s = shift;
	my %a = @_;

	if ( $a{title}       ne $s->title 	||
	     $a{description} ne $s->description || 
	     $a{url}         ne $s->url 	||
             $a{content}     ne $s->content 	) {
	     # IF ANY OF THIS IS DIFERENT THe MESSAGE
	     # HAS BEEN UPDATED
		print STDERR "Updating message ".$a{title}."\n";
		print STDERR "\nMy last version was : ".$s->{info}->{last_text};
		print STDERR "\nMy current version was : ".$s->text;
		$s->{info}->{last_raw_version} 	= $s->{info}->{raw}; 
		$s->{info}->{last_text} 	= $s->content || $s->description; 

		$s->{info}->{title} 		= $a{title};
		$s->{info}->{description} 	= $a{description};
		$s->{info}->{url} 		= $a{url};
		$s->{info}->{content} 		= $a{content};
		$s->{info}->{raw} 		= \%a;

		$s->{info}->{updated}      	= 1;
		$s->{info}->{last_updated} 	= time();

		$s->mark_unread();
		print STDERR "\nMy last version is noe".$s->{info}->{last_text};
	}

	$s->notify("message_updated");

	return 1;
}

sub get_feed {
    return RSSReader::Feed->new(id=>shift->{info}->{feed_id});
}

sub mark_read {
    my $s = shift;
    return if $s->{info}->{read};
    $s->{info}->{read} = 1;
    $s->notify('mark_read');
    $s->get_feed->_message_marked_read( message=> $s);
}

sub mark_unread {
    my $s = shift;
    return if (! $s->{info}->{read} );
    $s->{info}->{read} = 0;
    $s->notify('mark_unread');
    RSSReader->new->notify('message_marked_unread', message=> $s);
    $s->get_feed->_message_marked_unread( message=> $s);
}

sub delete {
    my $s = shift;
    $s->notify('message_deleted_before');
    $s->log_debug("The message ".$s->title." was deleted");
    my $config     = RSSReader::Configuration->new();
    my $del_result = $config->remove_message(feed_id => $s->get_feed->id(), id=>$s->id);
    if (!$del_result) {  return undef }
    $s->get_feed->_message_deleted(message=>$s);
    $s->notify('message_deleted');
    delete $_messages->{$s->id};
    return 1; 
}

sub read 	{ return shift->{info}->{read}; 	}
sub url   	{ return shift->{info}->{url};   	}
sub description	{ return shift->{info}->{description};	}
sub date	{ return shift->{info}->{date};		}
sub to_keep     { return shift->{info}->{to_keep};      }
sub fetch_time  { return shift->{info}->{fetch_time};   }

sub id { 
	my $s = shift;
	return $s->{id} || $s->{title}; 			
}

sub comments    { 
	my $s = shift;

	$s->log_debug($s->{info}->{comments});
	
	return $s->{info}->{comments} if $s->{info}->{comments};     

	return undef unless $s->get_feed->extra_info();

	if (($s->get_feed->extra_info()->{admin}) && 
	    ($s->get_feed->extra_info()->{admin}->{generatorAgent} =~ /movabletype/)) {
		my $id = undef;
		if ($s->{info}->{guid}) {
			$id = $s->{info}->{guid};
			$id = s/\@.*$//;
		} else {
			my $id_tmp = $s->url;	
			if ( $id_tmp =~ /.*\#(\d+)$/ ) {	
			 $id = $1;
			}
		}

		return undef unless $id;
		
		my $l = $s->get_feed->source_link().'/mt-comments.cgi?entry_id='.$id;
		$s->log_debug($l);
		return $l;
	}
	elsif (($s->get_feed->extra_info()->{admin}) &&
		($s->get_feed->extra_info()->{admin}->{generatorAgent} =~ /blogger\.com/)) {
		
	}
		

	return undef;
}

sub author      { 
	my $s = shift;
	
	return $s->{info}->{dc}->{creator} || $s->{info}->{author};       
}

sub content     { 
	my $s = shift;
	return $s->{info}->{content};      
}

sub category	{ 
	my $s = shift;
	my $categ = $s->{info}->{category} || $s->{info}->{dc}->{subject};
	return $categ;	
}

sub title { 
    my $s = shift;
    my %a = @_;
    my $t = $s->{info}->{title}; 	
    if ($a{escaped}) {
	$t = $s->_escape($t);
    }
    return $t;
}

sub text {
    my $s = shift;
    my %a = @_;
    my $text = $s->content || $s->description() || '';
    if ($a{escaped}) {
	$text = $s->_escape($text);
    }
    if ($ENV{SYNDIGATOR_DEBUG}) {
	$text = "ID: ".$s->id().$text;
    }
    return $text;
}

sub _escape {
    my $s = shift;
    my $t = shift;
    $t =~ s/^[\s\n]+//msg;
    $t =~ s/[\s\n]+$//msg;
    $t =~ s/&/&amp\;/msg; 
    return $t;
}

sub toggle_keep {
    my $s = shift;

    ($s->{info}->{to_keep} == 1 ) ? ($s->{info}->{to_keep} = 0) : ($s->{info}->{to_keep} = 1);

    $s->notify('keep_toggled');
    return $s->{info}->{to_keep};
}

sub mark_to_keep {
    my $s = shift;

    $s->{info}->{to_keep} = 1;
    $s->notify('keep_toggled');
    return $s->{info}->{to_keep};
}

sub mark_to_remove {
    my $s = shift;

    $s->{info}->{to_keep} = 0; 
    $s->notify('keep_toggled');
    return $s->{info}->{to_keep};
}

sub search {
    my $c = shift;
    my %a = @_;

    my @to_return= ();

    my $config    = RSSReader::Configuration->new();
    
    my $feed_id = undef;
    $feed_id = $a{feed}->id() if $a{feed};

    my @msgs = RSSReader::Configuration->new->search_messages(feed_id=>$feed_id,%a); 

    #print Dumper \@msgs;
    foreach my $message ( @msgs ) {
		next unless $message;
		my $new_obj = __PACKAGE__->new(id=>$message);
		if (exists $a{read}) { next if ($new_obj->read ne $a{read}) }
		push @to_return, $new_obj if $new_obj;
    }

    return wantarray ? @to_return : \@to_return;
    
}

1;
