package RSSReader::Fetcher::FindFeed;
use strict;
use RSSReader::Fetcher;
use base qw(RSSReader::Logger);

sub getFeedURL {
  my $s = shift;
  my %a = @_;

  my $url = $a{url};
  my $domain = $a{domain};
  my $feed_url;

  if ($url =~ /^aggregator:(.*)$/ 
     || $url =~ /^newsmonster-subscription:(.*)$/) {# agregator url
    $feed_url = $1;
  } elsif ($url =~ /^http:\/\/127\.0\.0\.1:\d+?\/.*(http:.*)$/) { #auto subscriber
    $feed_url = $1;
  } elsif ($url =~ /^http:\/\/.+?\/.*$/){
    $feed_url = $url;
  } elsif ($domain =~ /^(http:\/\/.+?)(\/.*\/).*?$/) {
    my $site = $1;
    my $path = $2;
    if ($url =~ /^\//) {
      $feed_url = $site.$url;
    } else {
      $feed_url = $site.$path.$url;
    }
  }
  elsif ($url=~ /^\/\//) {
  	$feed_url = 'http:'.$url;
  }
  elsif ($url !~ /^(http|https):\/\//) {
	$feed_url = 'http://'.$url;
  }
  else {
	$feed_url = $url;
  }

  return $feed_url;
}

sub getFeed {
  my $s = shift;
  my %a = @_;

  my $url = $a{url};
  my $is_feed;
  my ($success, $content, $resp) = RSSReader::Fetcher->get_url( $url );
  if ($success){
    $is_feed =  RSSReader::Fetcher->parse_feed( $content );
  }
  $s->log_debug("Is not feed") unless $is_feed;

  return ($is_feed, $content);
}

sub searchHtml4Feeds {
  my $s = shift;
  my %a = @_;

  my $html = $a{html};
  my @urls;

  @urls = $html =~ /<link\s+rel="alternate"\s+?type="application\/rss+xml"[^>]+?href="(.+?)"\s*\/?>/img;
  return @urls if scalar @urls;

  @urls = $html =~ /<link.+?type="application\/rss+xml".+?rel="alternate".+?href="(.+?)">/img;
  return @urls if scalar @urls;

  @urls = $html =~ /<link.+?rel="alternate"\s+?type="application\/rss+xml"[^>]+?href="(.+?)"\s*\/?>/img;
  return @urls if scalar @urls;

  @urls = $html =~ /\<LINK.+?REL=\"alternate\".+?HREF=\"(.+?)\".+?TYPE=\"application\/rss\+xml\".*?\>/img;
  return @urls if scalar @urls;

  @urls = $html =~ /\<LINK.+?REL=\"alternate\".+?TYPE=\"application\/rss\+xml\".+?HREF=\"(.+?)\".*?\>/img;
  return @urls if scalar @urls;

  return ();
}

sub run {
  my $s = shift;
  my %a = @_;

  my $url = $a{url};
  die "Missing param url" unless $url;
  $s->log_debug("Finding feed for $url");
 
  #Clean Url
  $url = $s->getFeedURL(url=>$url);
  $s->log_debug("The url , after clean up is : $url");

  # Get Page
  my ($is_feed, $content) = $s->getFeed(url=>$url);

  return { is_feed  => 1, 
           xml 	    => $is_feed,
	   url      => $url } if $is_feed;

  if ($content) {

    my @possible_feeds = $s->searchHtml4Feeds(html=>$content);

    foreach my $new_url (@possible_feeds) {

      $new_url = $s->getFeedURL(url=>$new_url, domain=>$url);

      # Get Page
      ($is_feed, $content) = $s->getFeed(url=>$new_url);

      $s->log_debug("got url: $new_url");
      return { is_feed  => 1, 
               xml      => $is_feed,
	       url     	=> $new_url } if $is_feed;
    }

  }
  return undef;
}

1;
