package RSSReader::Gtk2UI::AddAFeedToCategory;
use strict;
use RSSReader::Gtk2UI::Dialog;
use RSSReader::Gtk2UI::UrlTester;
use RSSReader::Gtk2UI::Preferences;

sub run {
  my $c = shift;
  my %a = @_;
  return undef unless $a{category};
  my $prefs =  RSSReader::Gtk2UI::Preferences->new;

  my $window = new Gtk2::Window('toplevel');
  $window->set_title("Create a new feed");
  $window->set_modal(1);
  $window->set_default_size(400,100);
  $window->set_transient_for( RSSReader::Gtk2UI::MainWindow->new()->widget );

  my $table = new Gtk2::Table(4,4,0);

  my $name_l= new Gtk2::Label('Name : ');
  $table->attach( $name_l    , 0, 1, 0, 1, ['fill', 'expand'], 'fill', 1 ,1 );
  my $name_e = new Gtk2::Entry(); 
  $table->attach( $name_e    , 1, 2, 0, 1, ['fill', 'expand'], 'fill', 1 ,1 );

  my $url_l= new Gtk2::Label('Url : ');
  $table->attach( $url_l    , 0, 1, 1, 2, ['fill', 'expand'], 'fill', 1 ,1 );
  my $url_e = new Gtk2::Entry( ); 
  $table->attach( $url_e    , 1, 2, 1, 2, ['fill', 'expand'], 'fill', 1 ,1 );
  my $url_b= new_with_label Gtk2::Button(' Test Url ');
  $table->attach( $url_b    , 2, 3, 1, 2, ['fill', 'expand'], 'fill', 1 ,1 );
  $url_b->set_sensitive(0) unless $prefs->online();
  
  my $f_interval_l = new Gtk2::Label('Fetch Interval (Min) : ');
  $table->attach( $f_interval_l , 0, 1, 2, 3, ['fill', 'expand'], 'fill', 1 ,1 );

  my $adj = new Gtk2::Adjustment( 60.0, 0.0, 30000.0, 1.0, 10.0, undef );
  my $f_interval_v = new Gtk2::SpinButton( $adj, '0.5', 0 );  
  $f_interval_v->set_sensitive(0); 
  $table->attach( $f_interval_v , 1, 2, 2, 3, ['fill', 'expand'], 'fill', 1 ,1 );
  my $f_interval_cb = new Gtk2::CheckButton( 'Use default' ); 
  $f_interval_cb->set_active(1);
  $f_interval_cb->signal_connect( 'toggled', sub {if ($f_interval_cb->get_active) { $f_interval_v->set_sensitive(0) } else { $f_interval_v->set_sensitive(1)}  } );
  $table->attach( $f_interval_cb , 2,3, 2, 3, ['fill', 'expand'], 'fill', 1 ,1 );
  my $al = new Gtk2::Alignment(0.5,0.5,0,0);
  $al->add($table);
  
  $table->show_all();

  my $ok_button     = Gtk2::Button->new_from_stock('gtk-ok');
  my $cancel_button = Gtk2::Button->new_from_stock('gtk-cancel');
  my $bbox = new Gtk2::HButtonBox();
  $bbox->add($ok_button);
  $bbox->add($cancel_button);
  $bbox->set_layout_default('end');
  $bbox->set_spacing_default( 3);
  $bbox->show_all;
  
  my $vbox = new Gtk2::VBox;
  $vbox->pack_start($al, 0,0,1);
  $vbox->pack_start(Gtk2::HSeparator->new,1,1,1);
  $vbox->pack_start($bbox,1,1,1);

  $window->add($vbox); 

  $window->show_all();

  my $s = bless { _widget 		=> $window, 
  		  _name_entry 		=> $name_e, 
		  _url_entry 		=> $url_e,  
		  _cat			=> $a{category}, 
		  _default_interval 	=> $f_interval_cb,
		  _fetch_interval	=> $f_interval_v
		}, $c;
  $ok_button->signal_connect('clicked', sub { $s->create_it() });
  $cancel_button->signal_connect('clicked', sub { $s->{_widget}->hide } );
  $url_b->signal_connect('clicked', sub { $s->_test_url() });
}

sub create_it {
  my $s = shift;

  my $interval;
  unless ($s->{_name_entry}->get_text()) {
  	RSSReader::Gtk2UI::Dialog->run('Please insert a name to the new feed'); 
	return;
  }
  unless ( $s->{_url_entry}->get_text() ) {
  	RSSReader::Gtk2UI::Dialog->run('Please insert a url to the new feed');
	return;
  }
  if ( $s->{_default_interval}->get_active ) {
	$interval = 0;
  } else {
	$interval = $s->{_fetch_interval}->get_value_as_int(); 
  }

  my $feed = RSSReader::Feed->create( 
  			   category 		=> $s->{_cat},
  			   url	    		=> $s->{_url_entry}->get_text(),
			   label    		=> $s->{_name_entry}->get_text(),
			   fetch_interval	=> $interval,
			   );
  unless ($feed) {
  	RSSReader::Gtk2UI::Dialog->run($@);
	return;
  }

  RSSReader::Gtk2UI->new->get_feed($feed) if RSSReader::Gtk2UI::Preferences->new()->online;

  $s->{_widget}->destroy();
  return;
}

sub _test_url {
        my $s = shift;
	if ( $s->{_url_entry}->get_text() !~ /^.*\:\/\/.*/ ) {
		$s->{_url_entry}->set_text('http://'.$s->{_url_entry}->get_text())
	}
	my $rss = RSSReader::Gtk2UI::UrlTester->run(url=>$s->{_url_entry}->get_text());
	if ($rss->{title}) {
		$s->{_name_entry}->set_text($rss->{title});
	}
}

1;

