package RSSReader::Gtk2UI::ExporterDruid;
use strict;
use RSSReader::Exporter;
use RSSReader::Gtk2UI::Dialog;
use Data::Dumper;
use base qw( RSSReader::Logger);

sub run {
    my $c        = shift;
    my $exporter = shift;

    my $w = new Gtk2::Window;
    $w->set_title("Syndigator ".$exporter->name()." exporter");
    my $druid = Gnome2::Druid->new(); 
    my $s = bless { _exporter => $exporter, _druid=>$druid}, $c;
    $s->{_druid} = $druid;
    $s->{_widget} = $w;

    my $start = Gnome2::DruidPageEdge->new('start'); 
    $start->set_title("Syndigator ".$exporter->name()." exporter: ");
    $start->set_text($exporter->description);
    $start->show();
    $druid->append_page($start);
    $druid->show();
    $druid->set_page($start);

    push @{$s->{_pages}}, $start;

    #info collecting
    my $page1 = Gnome2::DruidPageStandard->new();
    $page1->set_title("Information gathering");
    $page1->show();
    my $label_into = new Gtk2::Label("The following information is required to export you subscriptions:");
    $page1->vbox->pack_start($label_into,0,1,20); 
    $label_into->set_alignment(0.5,0.0);
    $label_into->show();

    my $page_1_table = new Gtk2::Table(20,2); 
    my $k = 0;
    foreach my $_opt ( sort(keys %{$exporter->config_options}) ) {
    	my $config_opt = $exporter->config_options->{$_opt};
	$c->log_debug("Found an option ".Dumper($config_opt) );	
	my $option_label = Gtk2::Label->new($config_opt->{desc}." : ");
	$option_label->set_alignment(1.0,0.5);
	$page_1_table->attach_defaults($option_label,0,1,$k, ($k+1));

	my $opt_widget;
	if ($config_opt->{type} eq 'text') { 
		my $entry = Gtk2::Entry->new();
		$page_1_table->attach_defaults($entry,1,2,$k, ($k+1));
		if ($config_opt->{default_value}) { $entry->append_text($config_opt->{default_value}) }
		$opt_widget = $entry;

	} elsif ( $config_opt->{type} eq 'file' ) {
		my $fentry = Gnome2::FileEntry->new(1,'Select '.$config_opt->{desc});
		if ($config_opt->{default_value}) { $fentry->gtk_entry->append_text($config_opt->{default_value}) }
		$opt_widget = $fentry;

	} elsif ( $config_opt->{type} eq 'boolean' ) {
		$opt_widget = Gtk2::CheckButton->new();
		if ($config_opt->{default_value}) { $opt_widget->set_active($config_opt->{default_value}) }
	}

	$page_1_table->attach_defaults($opt_widget,1,2,$k, ($k+1));
	$s->{_options_to_run}->{$_opt} = { %$config_opt };
	$s->{_options_to_run}->{$_opt}->{widget}=$opt_widget;
	$k++;

    }
    $page_1_table->show_all;
    $page1->vbox->pack_start($page_1_table,0,1,0);
    $page1->signal_connect('next', sub {return $_[2]->pre_export}, $s );
    $s->{_page_1} = $page1;
    $druid->append_page($page1);

    #PAGE FINAL
    my $pagefinal = Gnome2::DruidPageEdge->new('finish');;
    $pagefinal->set_text("Press 'Finish' to export your subscriptions.");
    $s->{_page_final} = $pagefinal;
    $s->{_page_final}->show_all;
    $s->{_page_final}->signal_connect('finish',  sub { $_[2]->export }, $s );
    $druid->append_page($pagefinal);

    $druid->signal_connect('cancel', sub { $w->destroy } );
    $s->{_current_page} = 0;

    $w->add($druid);
    $w->show();
}

sub pre_export {
    my $s = shift;
    #Get options and validate;
    foreach my $_opt ( keys %{$s->{_options_to_run}} ) {
	my $option = $s->{_options_to_run}->{$_opt};
    	$s->log_debug("Examing option ".$option->{desc});
	next unless $option->{mandatory};

	if ($option->{type} eq 'text') {
		unless ($option->{mandatory} && $option->{widget}->get_text()) {
			RSSReader::Gtk2UI::Dialog->run("Please fill the ".$option->{desc}." field"); 
			return 1;
		}
	}
	elsif ( $option->{type} eq 'file' ) {
		unless ( $option->{mandatory} && $option->{widget}->gtk_entry->get_text()) {
			RSSReader::Gtk2UI::Dialog->run("Please fill the ".$option->{desc}." field");
			return 1;
		}
		unless ( $option->{widget}->gtk_entry->get_text() =~ /^\// ) {
			my $e = $option->{widget}->gtk_entry;
			$e->set_text( $ENV{HOME}.'/'.$e->get_text );
		}
	}
    }
    $s->{_druid}->set_page($s->{_page_final});
    return 1;
    
}

sub export { 
     my $s = shift;
     my %a = @_;

     my %options;
     foreach my $_opt ( keys %{$s->{_options_to_run}} ) {
	my $option = $s->{_options_to_run}->{$_opt};

	my $val;
	if     ($option->{type} eq 'text')    { $val = $option->{widget}->get_text(); 			}
	elsif  ($option->{type} eq 'file' )   { $val = $option->{widget}->gtk_entry->get_text(); 	}
	elsif  ($option->{type} eq 'boolean') { $val = $option->{widget}->get_active 			}  

	$options{$_opt} = $val;
     }

     $s->log_debug("My options: ".Dumper(\%options));
     my $f = RSSReader::Feed->search();
     my $res = $s->{_exporter}->export(feeds=>$f, %options);
     if ($res) {
	$s->{_widget}->destroy;
	return 1;
     }
     else {
	RSSReader::Gtk2UI::Dialog->run($@);
	return 1;
     }
}

1;
