package RSSReader::Gtk2UI::Feed;
use RSSReader::Feed;
use RSSReader::Gtk2UI::Feed::InfoPane;
use RSSReader::Gtk2UI::Feed::MessageList;
use RSSReader::Gtk2UI::Feed::Properties;
use RSSReader::Gtk2UI::Message;
use RSSReader::Gtk2UI::Preferences;
use Data::Dumper;
use base qw(RSSReader::Logger);

our $_feeds;

sub new {
        my $c = shift;
        my %a = @_;

        if (exists $_feeds->{$a{id}}) { return $_feeds->{$a{id}}; }

        #otherwise .. fetch the cat and build the widget... after this
        # only redraws are needed.

        my $feed = RSSReader::Feed->new(id=>$a{id});
        my $s = bless { _feed => $feed, id => $a{id} }, $c;
        $s->{_prefs} = RSSReader::Gtk2UI::Preferences->new();

        $_feeds->{$a{id}} = $s;
        #TODO Register em updates da categoria, redraw proposals

        $s->{_widget} = new Gtk2::VPaned();

        #get handle_position
        my $position;
        if ($s->{_prefs}->per_view_pane()) {
            $position = $s->{_prefs}->get('feed_view_'.$s->{_feed}->id(), 'pane_heigth') || 350;
        }
        else {
            $position = $s->{_prefs}->get('all_views_global', 'pane_heigth') || 350;
        }
        $s->{_widget}->set_position($position);

        $s->{_message_selected} = undef;
	$s->{_message_widget}  = RSSReader::Gtk2UI::Message->new();

	$s->_redraw();
        $s->show();

        return $s;
}

sub get_feed { return shift->{_feed} }

sub show {
        my $s = shift;
	$s->{_message_list}->show;
        if ($s->{_prefs}->per_view_pane()) {
            $position = $s->{_prefs}->get('feed_view_'.$s->{_feed}->id(), 'pane_heigth') || 350;
        }
        else {
            $position = $s->{_prefs}->get('all_views_global', 'pane_heigth') || 350;
        }
	$s->widget->set_position($position);
        $s->widget()->show()
}

sub message_selected {
	my $s = shift;
	my %a = @_;
	$s->log_debug("I am feed widget and a message was selected");
	$s->{_message_widget}->load_message(message=>$a{message});
	$s->{_message_selected} =$a{message};
	$a{message}->mark_read();
}

sub widget {
        my $s = shift;
        return $s->{_widget};
}

sub id {
        my $s = shift;
        return $s->{id};
}

sub _redraw {
	my $s = shift;
        my $box = new Gtk2::VBox(0,2);
        $s->{_box} = $box;
	$s->{_widget}->add1($box);

        my $panel = RSSReader::Gtk2UI::Feed::InfoPane->new( id=>$s->{_feed}->id );
        $box->pack_start($panel->widget(),0,0,0);

	my $list  = RSSReader::Gtk2UI::Feed::MessageList->new( id=>$s->{_feed}->id );
	$box->pack_start($list->widget,1,1,1);
	$list->notify_me(event=>"message_selected", caller=>$s, call=>'message_selected');

	$box->show_all;
	$s->{_message_list} = $list;

	$s->{_widget}->add2($s->{_message_widget}->widget);
}

sub has_unread_messages  { 
	my $s = shift;
	my %a = @_;

	if ($a{scroll} && $s->{_message_selected}) {
		return 1 unless ( $s->{_message_widget}->full_scrolled() );
	}
	
	return scalar(@{$s->{_feed}->unread_messages}) 
}

sub select_next_unread   { 
	my $s = shift;
	my %a = @_;
	if ($a{scroll} && 
            $s->{_message_selected} && 
            (! $s->{_message_widget}->full_scrolled) ) {
		$s->{_message_widget}->scroll();
		return 1;
	}
	return $s->{_message_list}->select_next_unread(%a); 
}

sub can_fetch            { return 1 }
sub can_show_preferences { return 1 }

sub do_fetch             { RSSReader::Gtk2UI->new->get_feed(shift->get_feed) 		}
sub do_show_preferences  { RSSReader::Gtk2UI::Feed::Properties->run(shift->get_feed)    }

sub hide {
	my $s = shift;
	$s->{_message_list}->hide;
        $position = $s->{_prefs}->set('feed_view_'.$s->{_feed}->id(), 'pane_heigth', $s->{_widget}->get_position);

        if (! $s->{_prefs}->per_view_pane()) {
            $position = $s->{_prefs}->set('all_views_global', 'pane_heigth',$s->{_widget}->get_position);
        }
}


1;
