package RSSReader::Gtk2UI::GeneralProperties;
use strict;
use RSSReader::Configuration;
use RSSReader::Gtk2UI::Notifiers;
use RSSReader::Gtk2UI::Preferences;
use Gtk2::SimpleList;
use Data::Dumper;
use base qw(RSSReader::Logger);

sub run {
  my $c    = shift;

  my $conf  = RSSReader::Configuration->new();
  my $prefs = RSSReader::Gtk2UI::Preferences->new(); 

  my $w    = new Gnome2::App('_properties', "General preferences");
  my $vbox = new Gtk2::VBox; 
  $w->set_contents($vbox);
  $w->set_default_size( 400, 200 );
  $w->set_modal(1);

  my $notebook = new Gtk2::Notebook();
  $vbox->pack_start($notebook,1,1,1);
  $notebook->set_tab_pos('top');

  my $s = bless { }, $c;

  #PAGE 1;
  my $vbox_page1 = new Gtk2::VBox; 
  my $page1_label = new Gtk2::Label('General');
  $notebook->append_page( $vbox_page1, $page1_label);
    my $page_1_intro = Gtk2::Label->new();
    my $page_1_intro2 = Gtk2::Label->new();
    $page_1_intro->set_markup('<b>General options</b>');
    $page_1_intro2->set_markup('<small>Options that affect syndigator engine</small>');
    $vbox_page1->pack_start($page_1_intro,0,0,2);
    $vbox_page1->pack_start($page_1_intro2,0,0,2);


    # ZEEE TABLE

  
    my $table_page1 = new Gtk2::Table(5,5,0);
    my $table_page1_a = new Gtk2::Alignment(0.5,0.5,0,0);

    $s->{_fetch_int_l} = new Gtk2::Label('Default fetch interval (Mins) :');
    $s->{_fetch_int_l}->set_alignment(0.0,1.0);
    my $adj = new Gtk2::Adjustment( 1.0, 0.0, 36000, 1.0, 10.0, undef );
    $s->{_fetch_int_s} = new Gtk2::SpinButton( $adj, '0.5', 0 );
    $s->{_fetch_int_s}->set_value($prefs->option("fetch_interval"));
    $s->{_fetch_int_s}->signal_connect('changed', sub {$s->_activate_apply});

    $s->{_auto_save_l} = new Gtk2::Label('Data auto save :');
    $s->{_auto_save_l}->set_alignment(1.0,1.0);
    $s->{_auto_save_e} = new Gtk2::CheckButton(' enable');

    $s->{_auto_save_int_l} = new Gtk2::Label('Auto save interval (Mins):');
    $s->{_auto_save_int_l}->set_alignment(1.0,1.0);
    my $adj2 = new Gtk2::Adjustment( 1.0, 0.0, 36000, 1.0, 10.0, undef );
    $s->{_auto_save_int_s} = new Gtk2::SpinButton( $adj2, '0.5', 0 );
    $s->{_auto_save_int_s}->set_value($prefs->option("auto_save_interval"));
    $s->{_auto_save_int_s}->show();
    $s->{_auto_save_int_s}->set_sensitive(0);
    $s->{_auto_save_int_s}->signal_connect('changed', sub {$s->_activate_apply});

    $s->{_save_on_exit_l} = new Gtk2::Label('Save data on exit :');
    $s->{_save_on_exit_l}->set_alignment(1.0,1.0);
    $s->{_save_on_exit_e} = new Gtk2::CheckButton(' enable');
    if ($prefs->save_on_exit) { $s->{_save_on_exit_e}->set_active(1); }
    $s->{_save_on_exit_e}->signal_connect_after('toggled',sub {$s->_activate_apply});

    if ( $prefs->option("auto_save") ) { 
    		$s->{_auto_save_e}->set_active(1); 
		$s->{_auto_save_int_s}->set_sensitive(1);  
    }

    $table_page1->attach($s->{_fetch_int_l}      ,0,1,0,1,['fill','shrink'], 		[], 4,4);
    $table_page1->attach($s->{_fetch_int_s}      ,1,2,0,1,['fill','shrink','expand'], 	[], 4,4);
    $table_page1->attach($s->{_auto_save_l}      ,0,1,1,2,['fill','shrink'], 		[], 4,4); 
    $table_page1->attach($s->{_auto_save_e}      ,1,2,1,2,['fill','shrink'], 		[], 4,4);
    $table_page1->attach($s->{_auto_save_int_l}  ,0,1,2,3,['fill','shrink'], 		[], 4,4); 
    $table_page1->attach($s->{_auto_save_int_s}  ,1,2,2,3,['fill','shrink'], 		[], 4,4); 
    $table_page1->attach($s->{_save_on_exit_l}   ,0,1,3,4,['fill','shrink'],            [], 4,4);
    $table_page1->attach($s->{_save_on_exit_e}   ,1,2,3,4,['fill','shrink'],            [], 4,4);

    $table_page1->show_all();
    $table_page1_a->add($table_page1);

  $vbox_page1->pack_start($table_page1_a, 1,1,1);

  #PAGE 2;
  my $vbox_page2= new Gtk2::VBox;
  my $page2_label = new Gtk2::Label('Messages');
  #$notebook->append_page( $vbox_page2, $page2_label );


  #PAGE 3
  my $pane_page3 = new Gtk2::HPaned;
  $pane_page3->set_position(150);
  my $page3_label = new Gtk2::Label('Notifications');
  my $table_page3_1 = new Gtk2::Table(2,1,0);
  $notebook->append_page( $table_page3_1, $page3_label );
  my $list_label =  new Gtk2::Label();
  $list_label->set_markup('<b>Available Notifiers</b>');
  
  my $list_scrolled_win = new Gtk2::ScrolledWindow( undef, undef );
  $list_scrolled_win->set_policy( 'automatic', 'automatic' );
  my $list = new Gtk2::SimpleList(
				  "Id"   => 'hidden',
				  "Name" => "text",
				 );
  $list->show;
  $list_scrolled_win->add( $list );
  $list_scrolled_win->show_all();
     
  my @notifiers = RSSReader::Gtk2UI::Notifiers->get_notifiers();
  print STDERR Dumper(\@notifiers);	
  foreach (@notifiers) {
	print STDERR "NT: ".$_->name()." ".$_->id()."\n";
	push @{$list->{data}}, [ $_->id(), $_->name() ];
  }
  my $selection = $list->get_selection;
  $selection->signal_connect('changed', sub { $s->_notifier_selected() } );


  $table_page3_1->attach($list_label ,0,1,0,1,['fill','shrink','expand'], [], 2,2);
  $table_page3_1->attach($pane_page3 ,0,1,1,2,['fill','shrink','expand'], ['expand', 'fill'], 2,4);
  $table_page3_1->show_all;
  $s->{_notifier_pane_table}    = $table_page3_1; 
  $s->{_notifier_pane_list}     = $list;
  $s->{_notifier_pane}          = $pane_page3;
  $s->{_notifier_initial_label} = Gtk2::Label->new("Please select a notifier from the list."); 
 
  $pane_page3->add1($list_scrolled_win);
  $pane_page3->add2($s->{_notifier_initial_label});
  $pane_page3->show_all();

  #PAGE 4;
  my $vbox_page4 = new Gtk2::VBox; 
  my $page4_label = new Gtk2::Label('GUI');
  $notebook->append_page( $vbox_page4, $page4_label);
    my $page_4_intro = Gtk2::Label->new();
    my $page_4_intro2 = Gtk2::Label->new();
    $page_4_intro->set_markup('<b>GUI options</b>');
    $page_4_intro2->set_markup('<small>This options affect the way Syndigator displays the content</small>');
    $vbox_page4->pack_start($page_4_intro,0,0,2);
    $vbox_page4->pack_start($page_4_intro2,0,0,2);

    #ZEE TABLE
    my $table_page4   = new Gtk2::Table(5,5,0);
    my $table_page4_a = new Gtk2::Alignment(0.5,0.5,0,0);

    $s->{_same_column_width_l} = new Gtk2::Label('Use same column width for every view:');
    $s->{_same_column_width_l}->set_alignment(0.0,1.0);
    $s->{_same_column_width_e} = new Gtk2::CheckButton();
    $s->{_same_column_width_e}->set_active(1) unless $prefs->option('per_view_columns');
    $s->{_same_column_width_e}->signal_connect_after('toggled',sub {$s->_activate_apply});
	
    $s->{_same_pane_size_l} = new Gtk2::Label('Use same pane size for every view:');
    $s->{_same_pane_size_l}->set_alignment(0.0,1.0);
    $s->{_same_pane_size_e} = new Gtk2::CheckButton();
    $s->{_same_pane_size_e}->set_active(1) unless $prefs->option('per_view_pane');
    $s->{_same_pane_size_e}->signal_connect_after('toggled',sub {$s->_activate_apply});

    $s->{_category_col_l} = new Gtk2::Label('Show the "Category" column in lists:');
    $s->{_category_col_l}->set_alignment(0.0,1.0);
    $s->{_category_col_e} = new Gtk2::CheckButton();
    $s->{_category_col_e}->set_active(1) if $prefs->option('show_category_col');
    $s->{_category_col_e}->signal_connect_after('toggled',sub {$s->_activate_apply});

    $s->{_colored_difs_l} = new Gtk2::Label('Show colored differences on updated messages:');
    $s->{_colored_difs_l}->set_alignment(0.0,1.0);
    $s->{_colored_difs_e} = new Gtk2::CheckButton();
    $s->{_colored_difs_e}->set_active(1) if $prefs->option('colored_diffs');
    $s->{_colored_difs_e}->signal_connect_after('toggled',sub {$s->_activate_apply});

    $table_page4->attach($s->{_same_column_width_l} ,0,1,0,1,['fill','shrink'], 		[], 4,4);
    $table_page4->attach($s->{_same_column_width_e} ,1,2,0,1,['fill','shrink'], 		[], 4,4);
    $table_page4->attach($s->{_same_pane_size_l}    ,0,1,1,2,['fill','shrink'], 		[], 4,4);
    $table_page4->attach($s->{_same_pane_size_e}    ,1,2,1,2,['fill','shrink'], 		[], 4,4);
    $table_page4->attach($s->{_category_col_l}      ,0,1,2,3,['fill','shrink'], 		[], 4,4);
    $table_page4->attach($s->{_category_col_e}      ,1,2,2,3,['fill','shrink'], 		[], 4,4);
    $table_page4->attach($s->{_colored_difs_l}      ,0,1,3,4,['fill','shrink'], 		[], 4,4);
    $table_page4->attach($s->{_colored_difs_e}      ,1,2,3,4,['fill','shrink'], 		[], 4,4);

    $table_page4->show_all();
    $table_page4_a->add($table_page4);


  $vbox_page4->pack_start($table_page4_a, 1,1,3);

  #PAGE 5
  my $vbox_page5  = new Gtk2::VBox;
  my $page5_label = new Gtk2::Label('Proxy');
  $notebook->append_page($vbox_page5, $page5_label); 
 
     #Zee desc label.
     my $page5_t_label = new Gtk2::Label();
     $vbox_page5->pack_start($page5_t_label, 0,0,2);
     $page5_t_label->set_markup("<b>Proxy options</b>");
     my $page5_label = new Gtk2::Label();
     $page5_label->set_markup("<small>When enabled, this option causes all connections to pass trough the specified proxy address.</small>");
     $vbox_page5->pack_start($page5_label, 0,0,2);


     $s->{_page_5_proxy_enable_a} = new Gtk2::Alignment(0.5,0.5,0,0);
     $s->{_page_5_proxy_enable} = new Gtk2::CheckButton(' Enable proxy support');
     $s->{_page_5_proxy_enable_a}->add($s->{_page_5_proxy_enable});
     $vbox_page5->pack_start($s->{_page_5_proxy_enable_a}, 0,1,10);
     $s->{_page_5_proxy_enable}->set_active(1) if $prefs->option('http_proxy_enabled');

     #ZEE table
     my $table_page5 = new Gtk2::Table(5,5,0);
     my $table_page5_a = new Gtk2::Alignment(0.5,0.5,0,0);
     $table_page5_a->add($table_page5);
     $s->{_page_5_proxy_host_l} = new Gtk2::Label('Proxy address: ');
     $s->{_page_5_proxy_host_l}->set_alignment(0.0,1.0);
     $s->{_page_5_proxy_host_e} = new Gtk2::Entry();
     $s->{_page_5_proxy_host_e}->set_sensitive(0) unless $prefs->option('http_proxy_enabled');
     $s->{_page_5_proxy_host_e}->set_text($prefs->option('http_proxy_host'));
     $s->{_page_5_proxy_host_e}->signal_connect("changed", sub { $s->_activate_apply } );

     $s->{_page_5_proxy_port_l} = new Gtk2::Label('Proxy port: ');
     $s->{_page_5_proxy_port_l}->set_alignment(0.0,1.0);
     $s->{_page_5_proxy_port_e} = new Gtk2::Entry();
     $s->{_page_5_proxy_port_e}->set_sensitive(0) unless $prefs->option('http_proxy_enabled');
     $s->{_page_5_proxy_port_e}->set_text($prefs->option('http_proxy_port'));
     $s->{_page_5_proxy_port_e}->signal_connect("changed", sub { $s->_activate_apply } );

     $table_page5->attach($s->{_page_5_proxy_host_l}      ,0,1,0,1,['shrink'], 		[], 4,4);
     $table_page5->attach($s->{_page_5_proxy_host_e}      ,1,2,0,1,['shrink'], 		[], 4,4);
     $table_page5->attach($s->{_page_5_proxy_port_l}      ,0,1,1,2,['shrink'], 		[], 4,4);
     $table_page5->attach($s->{_page_5_proxy_port_e}      ,1,2,1,2,['shrink'], 		[], 4,4);
     $vbox_page5->pack_start($table_page5_a, 0,1,20);

     $s->{_page_5_proxy_enable}->signal_connect_after('toggled',sub { $s->_activate_apply;
								      if ($s->{_page_5_proxy_enable}->get_active) {
									$s->{_page_5_proxy_host_e}->set_sensitive(1);
									$s->{_page_5_proxy_port_e}->set_sensitive(1);
								      } else {
									$s->{_page_5_proxy_port_e}->set_sensitive(0);
									$s->{_page_5_proxy_host_e}->set_sensitive(0);
								      }
     								     });

  $vbox_page5->show_all;


  #GLOBAL SETTNGS
  $s->{_apply_page1} = Gtk2::Button->new_from_stock('gtk-apply'); 
  $s->{_apply_page1}->set_sensitive(0);
  $s->{_apply_page1}->signal_connect('clicked', sub { $s->_apply_changes });
  $s->{_close_page1} = Gtk2::Button->new_from_stock('gtk-close');
  $s->{_close_page1}->signal_connect('clicked', sub { $w->destroy });
  $s->{_button_box_page1} = new Gtk2::HButtonBox();
  $s->{_button_box_page1}->add( $s->{_apply_page1} );
  $s->{_button_box_page1}->add( $s->{_close_page1} );
  $s->{_button_box_page1}->set_layout_default('end');
  $vbox->pack_start($s->{_button_box_page1}, 0,0,0);
  $s->{_auto_save_e}->signal_connect_after('toggled',
    			sub{
				$s->_activate_apply;
				if ($s->{_auto_save_e}->get_active) {
					$s->{_auto_save_int_s}->set_sensitive(1) 
				} else {
					$s->{_auto_save_int_s}->set_sensitive(0)
				}
			}
    );
  $w->show_all();
}

sub _notifier_selected {
  my $s = shift;
  my %a = @_;
  my ($idx) = $s->{_notifier_pane_list}->get_selected_indices;
  print STDERR "Item width index $idx was selected\n";

  my $n_id = @{$s->{_notifier_pane_list}->{data}}->[$idx]->[0];
  print STDERR "Notifier $n_id was selected\n";
 
  my $n = RSSReader::Gtk2UI::Notifiers->get_notifier($n_id);

  if ( exists $s->{_current_notifier} ) {
    $s->{_notifier_pane}->remove( $s->{_notifiers}->{$s->{_current_notifier}}->{widget} );
  }
  else {
    $s->{_notifier_pane}->remove($s->{_notifier_initial_label});
  }

  $s->{_current_notifier} = $n->id();

  if (  exists $s->{_notifiers}->{$n->id()}  ) {
    $s->{_notifier_pane}->add2($s->{_notifiers}->{$n->id()}->{widget});
    return 1;
  }


  my $hbox  = new Gtk2::VBox(9);
  my $table = new Gtk2::Table(3,2,0);
  my $desc  = new Gtk2::Label($n->description);
  $desc->set_line_wrap(1);
  $desc->set_alignment(0.5,0.0);
  $table->attach($desc  ,0,2,0,1,['fill','shrink','expand'], [], 2,4);
  $s->{_notifiers}->{$n->id()}->{widget} = $hbox;

  if ($n->is_active) {

    #the enable;
    my $enable = new Gtk2::CheckButton(' Enable this notifier');
    $enable->set_active(0);
    $enable->set_active(1) if $n->is_enabled;
    $enable->signal_connect('toggled', sub { $s->_activate_apply });
    $s->{_notifiers}->{$n->id()}->{enable}->{widget} = $enable;
    $table->attach($enable  ,1,2,1,2,['fill','shrink','expand'], [], 2,4);

  #the options;
  if (scalar(@{$n->config_options})) {
   my $frame = new Gtk2::Frame(" Options "); 
   $frame->show();
   $table->attach($frame  ,0,2,2,3, ['fill','shrink','expand'], ['expand','fill'], 2,4);
   my $box = Gtk2::VBox->new(0,2);
   $box->show_all();
   $frame->add($box);

   foreach my $option (@{$n->config_options}) {
	my $b = Gtk2::HBox->new(0,2);
	my $o_label = Gtk2::Label->new(" ".$option->{label}." : ");

	if ( $option->{type} eq 'spin integer' ) {
	#SPIN INTEGERS
		my $opt = Gtk2::SpinButton->new_with_range (0, 999, 1);
		$s->{_notifiers}->{$n->id()}->{option}->{$option->{name}}->{widget} = $opt;
		if ( $n->get_option($option->{name}) ) {
		   $opt->set_value( $n->get_option($option->{name}) );
		}
		if ( $option->{default} ) { 
		   $opt->set_value($option->{default});
		}
		$s->{_notifiers}->{$n->id()}->{option}->{$option->{name}}->{value} = 
				sub {
				  $s->{_notifiers}->{$n->id()}->{option}->{$option->{name}}->{widget}->get_value()
				}; 
		$opt->signal_connect('value-changed', sub { $s->_activate_apply });
	}
	$b->pack_start($o_label,0,0,1);
	$b->pack_start($s->{_notifiers}->{$n->id()}->{option}->{$option->{name}}->{widget},0,0,1);
	$box->pack_start($b,0,0,1);	
   }
   $frame->show_all();
   
  }
 

  } #ELSE FAILED TO LOAD
  else {
    my $label = Gtk2::Label->new( $n->fail_reason );     
    my $err_label = Gtk2::Label->new( "This notifier cannot be used" );
    my $img = Gtk2::Image->new_from_stock("gtk-dialog-warning",'small-toolbar');
    $img->show;
    $label->set_line_wrap(1);
    $label->set_alignment(0.5,0.0);
    $label->show;
    $table->attach($img        ,1,2,1,2,['fill','shrink','expand'], [], 2,4);
    $table->attach($err_label  ,1,2,2,3,['fill','shrink','expand'], [], 2,4);
    $table->attach($label      ,1,2,3,4,['fill','shrink','expand'], [], 2,4);
  }

  $table->show_all();
  $hbox->pack_start($table, 1,1,1);
  $hbox->show_all();
  $s->{_notifier_pane}->add2($hbox);
}

sub _activate_apply {
  my $s = shift;
  $s->{_apply_page1}->set_sensitive(1);
  return 1;
}

sub _apply_changes {
  my $s = shift;

  my $conf  = RSSReader::Configuration->new();
  my $prefs = RSSReader::Gtk2UI::Preferences->new(); 

  if ( $s->{_auto_save_e}->get_active && ($s->{_auto_save_int_s}->get_value_as_int() == 0) ) {
	RSSReader::Gtk2UI::Dialog->run('You cannot save you data every 0 minutes');
	return;
  }
  if ($s->{_fetch_int_s}->get_value_as_int() == 0) {
	RSSReader::Gtk2UI::Dialog->run('You cannot fetch new data every 0 minutes');
	return;
  }

  if ($s->{_page_5_proxy_enable}->get_active && (!$s->{_page_5_proxy_host_e}->get_text)) {
	RSSReader::Gtk2UI::Dialog->run('Please insert a valid proxy host');
	return;
  }

  if ($s->{_page_5_proxy_enable}->get_active && (!$s->{_page_5_proxy_port_e}->get_text)) {
	RSSReader::Gtk2UI::Dialog->run('Please insert a valid proxy port');
	return;
  }

  #AND NOW CHANGED NOTIFIERS 
  #TODO probably verify it before applying
  foreach my $id (keys %{$s->{_notifiers}}) {
  	my $n = RSSReader::Gtk2UI::Notifiers->get_notifier($id);
	$s->log_debug("Notifier $id was modified");
	print STDERR "Analysing notifier $id\n";
	next unless $n->is_active;
	if ($s->{_notifiers}->{$id}->{enable}->{widget}->get_active) {
		$n->enable;
	} else {
		$n->disable;
	}

	foreach my $o (keys %{$s->{_notifiers}->{$id}->{option}} ) {
		print STDERR "  Checking option $o\n";
		my $v = $s->{_notifiers}->{$id}->{option}->{$o}->{value}->();
		print STDERR "  Has value $v\n";
	
		$n->set_option($o,$v);
	}
  }

  $prefs->option("auto_save"         ,$s->{_auto_save_e}->get_active);
  $prefs->option("save_on_exit"      ,$s->{_save_on_exit_e}->get_active);
  $prefs->option("auto_save_interval",$s->{_auto_save_int_s}->get_value_as_int());
  $prefs->option("fetch_interval"    ,$s->{_fetch_int_s}->get_value_as_int());
  $prefs->option('http_proxy_enabled',$s->{_page_5_proxy_enable}->get_active);
  $prefs->option('http_proxy_host'   ,$s->{_page_5_proxy_host_e}->get_text);
  $prefs->option('http_proxy_port'   ,$s->{_page_5_proxy_port_e}->get_text);

  #PAGE 4 - GUI Options
  $prefs->option('per_view_columns', (!$s->{_same_column_width_e}->get_active) );
  $prefs->option('per_view_pane',    (!$s->{_same_pane_size_e}->get_active)    );
  $prefs->option('show_category_col',  $s->{_category_col_e}->get_active );
  $prefs->option('colored_diffs',      $s->{_colored_difs_e}->get_active );

  $s->{_apply_page1}->set_sensitive(0);
}

1;
