package RSSReader::Gtk2UI::Launcher::SearchLauncher;

use strict;
use Data::Dumper;
use RSSReader::Gtk2UI::Dialog;
use RSSReader::Gtk2UI::Search;
use base ('RSSReader::Logger', 'RSSReader::Gtk2UI::Launcher');

sub new { return bless {}, shift };

sub run {
    my $s = shift;
    my %a = @_;

    $s->{_step}           = 0;
    $s->{_display_widget} = undef;
    $s->{_display_area}   = new Gtk2::VBox;
    $s->{vbox} 		  = new Gtk2::VBox;
    $s->{vbox}->pack_start($s->{_display_area},1,1,1);
    $s->{vbox}->pack_start(Gtk2::HSeparator->new,0,0,0);
    $s->{vbox}->show_all();

    $s->{_button_box} = new Gtk2::HButtonBox();
    $s->{_button_box}->set_layout_default('end');
    $s->{_button_box}->set_spacing_default( 3);
    $s->{_button_box}->show;

    $s->{_go_button} = Gtk2::Button->new_from_stock('gtk-find');
    $s->{_go_button}->signal_connect('clicked', sub { $s->start_search() } );
    $s->{_button_box}->add($s->{_go_button});

    $s->{_cancel_button} = Gtk2::Button->new_from_stock('gtk-cancel');
    $s->{_cancel_button}->signal_connect('clicked', sub { $s->{_window}->destroy } );
    $s->{_button_box}->add($s->{_cancel_button});

    $s->{vbox}->pack_start($s->{_button_box},0,0,0);

    #Options
    my $f_box = Gtk2::HBox->new();

    #FIELDS
    my $f_omenu = Gtk2::OptionMenu->new();
    $s->{_f_omenu} = $f_omenu;
    my $f_menu = Gtk2::Menu->new();

    my $title_item = Gtk2::MenuItem->new_with_label("Story Title");
    my $text_item  = Gtk2::MenuItem->new_with_label("Story Text");

    $f_menu->append($title_item);
    $f_menu->append($text_item);
    $f_omenu->set_menu($f_menu);
    $f_box->pack_start($f_omenu, 0,0,0);

    #Contains
    my $om_ctns= Gtk2::OptionMenu->new();
    $s->{_ctns_omenu} = $om_ctns;
    my $om_menu = Gtk2::Menu->new();

    my $ctns_item = Gtk2::MenuItem->new_with_label("contains");
    $ctns_item->{"append"=> ""};

    my $dctns_item = Gtk2::MenuItem->new_with_label("does not contain");
    $dctns_item->{"append"=> "_without" };

    $om_menu->append($ctns_item);
    $om_menu->append($dctns_item);
    $om_ctns->set_menu($om_menu);
    $f_box->pack_start($om_ctns, 0,0,0);

    $s->{_t_entry} = new Gtk2::Entry;
    $f_box->pack_start($s->{_t_entry},0,0,0);
    
    #Options
    my $status_box = Gtk2::HBox->new();

    my $status_label = Gtk2::Label->new("Status :");
    $status_box->pack_start($status_label, 0,0,5);

    my $status_omenu = Gtk2::OptionMenu->new();
    $s->{_status_omenu} = $status_omenu;
    my $status_menu = Gtk2::Menu->new();

    my $any_item = Gtk2::MenuItem->new_with_label("Any");
    my $read_item = Gtk2::MenuItem->new_with_label("Read");
    my $unread_item  = Gtk2::MenuItem->new_with_label("Unread");

    $status_menu->append($any_item);
    $status_menu->append($read_item);
    $status_menu->append($unread_item);
    $status_omenu->set_menu($status_menu);
    $status_box->pack_start($status_omenu, 0,0,0);

    $s->{_display_area}->pack_start($f_box, 0, 0, 10);
    $s->{_display_area}->pack_start($status_box, 0, 0, 2);

    #Title
    $s->{_window} = new Gtk2::Window("toplevel");
    $s->{_window}->set_default_size( 250, 150 );
    $s->{_window}->set_title("Search Messages");

    $s->{_window}->add($s->{vbox});

    $s->{_window}->show_all;
}

sub start_search {
    my $s = shift;

    if ( $s->{_t_entry}->get_text =~ /^(\s+)?$/) {
	RSSReader::Gtk2UI::Dialog->run("Please insert some text to search");	
        return;
    }

    #FIRST OFF ALL.. lets validate user choices
    my %search_options = ();
    print STDERR "Field : ".$s->{_f_omenu}->get_history()."\n";
    print STDERR "Content : ".$s->{_ctns_omenu}->get_history()."\n";
    if ( $s->{_f_omenu}->get_history() == 0 && $s->{_ctns_omenu}->get_history==0 ) {
	$search_options{title} = $s->{_t_entry}->get_text; 
    }
    elsif ( $s->{_f_omenu}->get_history() == 1 && $s->{_ctns_omenu}->get_history==0 ) {
	$search_options{content}     = $s->{_t_entry}->get_text;
	$search_options{description} = $s->{_t_entry}->get_text; 
    }
    elsif ( $s->{_f_omenu}->get_history() == 0 && $s->{_ctns_omenu}->get_history==1 ) {
	$search_options{title_without} = $s->{_t_entry}->get_text;
    }
    elsif ( $s->{_f_omenu}->get_history() == 1 && $s->{_ctns_omenu}->get_history==1 ) {
	$search_options{content_without}     = $s->{_t_entry}->get_text; 
	$search_options{description_without} = $s->{_t_entry}->get_text; 
    }

    if ( $s->{_status_omenu}->get_history() == 1 ) {
	$search_options{read} = 1;
    } elsif ( $s->{_status_omenu}->get_history() == 2 ) {
	$search_options{read} = 0;
    }

    print STDERR Dumper(\%search_options);

    $s->run_search(search_options=>\%search_options);
}

sub run_search {
    my $s = shift;
    my %a = @_;
    $s->{_window}->remove($s->{vbox});

    $s->{searching} = 1;
    $s->{canceled}  = 0;


    $s->{searching_vbox}            = new Gtk2::VBox;
    $s->{_searching_display_area}   = new Gtk2::VBox;
    $s->{searching_vbox}->pack_start($s->{_searching_display_area},1,1,1);
    $s->{searching_vbox}->pack_start(Gtk2::HSeparator->new,0,0,0);

    $s->{_searching_button_box} = new Gtk2::HButtonBox();
    $s->{_searching_button_box}->set_layout_default('end');
    $s->{_searching_button_box}->set_spacing_default( 3);
    $s->{_searching_button_box}->show;

    $s->{_searching_cancel_button} = Gtk2::Button->new_from_stock('gtk-cancel');
    $s->{_searching_cancel_button}->signal_connect('clicked', sub { $s->{canceled}=1 } );
    $s->{_searching_button_box}->add($s->{_searching_cancel_button});

    $s->{searching_vbox}->pack_start($s->{_searching_button_box},0,0,0);

    #visible things
    $s->{_searching_display_area}->pack_start(Gtk2::Label->new("Please wait.. building message list"), 1,1,1);
    $s->{_progress_bar} = Gtk2::ProgressBar->new();
    $s->{_searching_display_area}->pack_start($s->{_progress_bar},0,0,10);

    $s->{_window}->add($s->{searching_vbox});
    $s->{_window}->show_all;


    my @m = RSSReader::Feed::Message->search(
						%{$a{search_options}},
				   		cancel_callback=> sub { return $s->{canceled} },
						idle_callback  => sub { 
								   Gtk2->main_iteration while ( Gtk2->events_pending );
								   shift->{_progress_bar}->pulse() 
								},
						user_data      =>$s
					   	); 

   if ( $s->{canceled} ) {
	print STDERR "Canceled\n";
	$s->{_window}->destroy;
	return;
   }

   foreach my $ms (@m) {
	print STDERR "Returned ".$ms."\n";
   }	

   #CREATE THE VIEW AND SHOW IT
   my $sv = RSSReader::Gtk2UI::Search->new(search_options=>$a{search_options}, search_results=>\@m);
   RSSReader::Gtk2UI::MainWindow->new->_change_view(view=>$sv);
   $s->{_window}->destroy;
}

sub attach {
	my $s = shift;
	my %a = @_;
	
	$a{toolbar}->append_item( 'Search', 'Search', 'Search',
                                   Gtk2::Image->new_from_stock('gtk-find','large-toolbar'),
                                                            sub { my ($b, $s) = @_; $s->run() }, $s
                                                            );

	print STDERR Dumper($a{menubar}->get_item("/Actions"));
	print STDERR Dumper(\%a);
}

### REGISTRING STUFF ####
sub id { return "search_launcher" }

RSSReader::Gtk2UI::Launcher->register( id => __PACKAGE__->id, package => __PACKAGE__ );

1;
