package RSSReader::Gtk2UI::MainWindow;
use strict;
use Data::Dumper;
use RSSReader;
use RSSReader::Gtk2UI::Launcher;
use RSSReader::Gtk2UI::Preferences;
use RSSReader::Gtk2UI::Tree;
use RSSReader::Gtk2UI::Feed;
use RSSReader::Gtk2UI::Category;
use RSSReader::Gtk2UI::Dialog;
use RSSReader::Gtk2UI::AddACategory;
use RSSReader::Gtk2UI::GeneralProperties;
use RSSReader::Gtk2UI::AddAFeedToCategory;
use RSSReader::Gtk2UI::DiscoverFeed;
use RSSReader::Gtk2UI::SaveConfiguration;
use RSSReader::Gtk2UI::IconFactory;
use RSSReader::Importer;
use RSSReader::Gtk2UI::ExporterDruid;
use RSSReader::Exporter;
use RSSReader::Gtk2UI::ImporterDruid;
use Gtk2::Gdk::Keysyms;
use base qw(RSSReader::Logger RSSReader::Gtk2UI);

our $_main_window;

sub new {
	my $c = shift;
	return $_main_window if defined($_main_window);	
	
	$_main_window = bless {}, $c;
	$_main_window->_init() unless $_main_window->{_init_done};

	return $_main_window;
}

sub show {
	my $s = shift;
	$s->_init() unless $s->{_init_done};
	$s->{_widget}->show();

	return;
}

sub hide {
	my $s = shift;
	$s->_init() unless $s->{_init_done};
	$s->{_widget}->hide();

	return;
}

sub widget { return shift->{_widget} }

sub _next_unread {
        my $s = shift;
        my %a = @_;
                                                                                                                                                          
        if ($s->{_current_view} && $s->{_current_view}->has_unread_messages(%a)) {
                $s->log_debug("selected unread message in current view");
                $s->{_current_view}->select_next_unread(%a);
        }
        else {
                $s->log_debug("selected unread message crawling to the tree");
                my $res = $s->{_tree}->select_next_unread(%a);
                if ($res) {
                        $s->log_debug("Got unread item :".$res->label);
                        $s->{_current_view}->select_next_unread(%a);
                } else {
                        $s->log_debug("No new messages");
                        RSSReader::Gtk2UI::Dialog->run("There are no unread messages", 'info') unless $a{quiet};
                }
        }
        return 1;
}

sub _init {
	my $s = shift;
	$s->{_widget} = Gnome2::App->new("Syndigator $VERSION", "Syndigator $VERSION");
	$s->{_widget}->signal_connect(delete_event=>sub {RSSReader::Gtk2UI->new->_exit} );
	my $prefs = RSSReader::Gtk2UI::Preferences->new();
        my $w_width  = $prefs->get('main_window', 'width')  || 800;
        my $w_height = $prefs->get('main_window', 'heigth') || 600;
	$s->{_widget}->set_icon(RSSReader::Gtk2UI::IconFactory->pixbuf('syndigator-logo-simple'));

	$s->{_widget}->set_default_size( $w_width, $w_height );
	$s->{_vbox} = Gtk2::VBox->new;
	$s->{_widget}->set_contents($s->{_vbox});

	$s->{_pane} = new Gtk2::HPaned();
	my $handle_pos = $prefs->get('main_window','gutter_pos') || 220;
	$s->{_vbox}->pack_start( $s->{_pane},1,1,0 );
	$s->{_pane}->set_position($handle_pos); 
	$s->{_pane}->show;

	$s->_init_toolbar;
	$s->_init_status_bar;
	$s->_init_menubar();

	#INITING LAUNCHERS
	foreach my $l (@{RSSReader::Gtk2UI::Launcher->get_launchers}) {
		$l->attach( menubar => $s->{_factory}, toolbar => $s->{_toolbar} );
	}

	my $tree = RSSReader::Gtk2UI::Tree->new();
	$tree->notify_me(event => 'view_selection',     caller=>$s, call=>'_change_view');
	$s->{_tree} = $tree;
	$s->{_pane}->pack1( $tree->widget(),0,1 );
	$tree->show();

	$s->_redraw_ui;

	RSSReader::Gtk2UI->new()->notify_me(event=>'exit', caller => $s, call => '_exiting');
	RSSReader::Gtk2UI::Preferences->new()->notify_me(event=>'online_changed', caller=> $s, call=>'_redraw_ui');

	$s->{_vbox}->show_all();
	$s->{_widget}->signal_connect('key-press-event', sub { $_[2]->_key_press(@_) }, $s);
	$s->{_init_done}=1;

}

sub _key_press {
   my $s = shift;
   my ($w, $event) = @_;

   print STDERR "Key ".$event->keyval." was ".$event->type."\n";
   if ($event->keyval == 32 ) {
	$s->_next_unread(quiet=>1, scroll=>1);
	return 1;

   } elsif ($event->keyval == 110) {
	$s->_next_unread(quiet=>1);
	return 1;
   }
   return undef;
}

sub _change_view {
   my $s = shift;
   my %a = @_;
   my $new_view = $a{view};

   $s->notify("view_changed_before");

   if ( exists($s->{_current_view}) && defined($s->{_current_view}) ) {
   	$s->{_current_view}->hide();
	$s->{_pane}->remove($s->{_current_view}->widget);		
   }

   $s->{_current_view}      = $new_view;
   $s->{_pane}->add2($s->{_current_view}->widget);
   $new_view->show();
   $s->{_current_view_type} = ref($new_view);

   $s->notify("view_changed");

   $s->_redraw_ui;
}

sub current_view {
   my $s = shift;
   return $s->{_current_view};
}

sub status_bar_add {
	my $s = shift;
        my $mid =  $s->{_status_bar}->push( $s->{_cont_id}, $_[0] );
        return $mid;
}

sub status_bar_remove {
        my $s = shift;
        $s->{_status_bar}->remove( $s->{_cont_id}, $_[0] );
        return 1;
}

sub _redraw_ui {
	my $s = shift;
	print STDERR "Online was toggled, dealing with it\n";
	if (RSSReader::Gtk2UI::Preferences->new()->online) {
		$s->{_factory}->get_item('/Actions/Fetch all feeds')->set_sensitive(1);
		$s->{_factory}->get_item('/Actions/Work offline')->show();
		$s->{_factory}->get_item('/Actions/Work online')->hide();

	} else {
		$s->{_factory}->get_item('/Actions/Fetch all feeds')->set_sensitive(0);
		$s->{_factory}->get_item('/Actions/Work offline')->hide();
		$s->{_factory}->get_item('/Actions/Work online')->show();

	}
	
	#VIEW SPECIFIC
   	if ( ($s->{_current_view}->can_fetch()) && (RSSReader::Gtk2UI::Preferences->new()->online()) ) {
   		$s->{_factory}->get_item('/Actions/Fetch current')->set_sensitive(1);
		$s->{_toolbar_fetch}->set_sensitive(1);
   	} else {
   		$s->{_factory}->get_item('/Actions/Fetch current')->set_sensitive(0);
		$s->{_toolbar_fetch}->set_sensitive(0);
   	}

	if ($s->{_current_view}->can_show_preferences) {
		$s->{_toolbar_prefs}->set_sensitive(1);
	}
	else {
		$s->{_toolbar_prefs}->set_sensitive(0);
	}

	$s->_redraw_online_button;
	return 1;
}
sub _redraw_online_button {
        my $s = shift;
        if (! $s->{_online_button}) {
		print STDERR "There was no online button\n";
                $s->{_online_button} = Gtk2::Button->new();
		$s->{_online_button}->show;
                $s->{_online_button}->signal_connect('clicked', sub {
                                                          my $conf = RSSReader::Gtk2UI::Preferences->new;
                                                          if ($conf->online) {
                                                                $conf->online(0);
                                                          } else {
                                                                $conf->online(1);
                                                          }
                                                        } );
                $s->{_online_button}->set_relief('none');
                $s->{_online_button}->show;
                $s->{pix} = Gtk2::Image->new_from_file(RSSReader::Gtk2UI::IconFactory->filename('online'));
                $s->{pix}->show();
                $s->{_online_button}->add($s->{pix});
                $s->{_online_button_tooltips} = Gtk2::Tooltips->new;
                $s->{_status_hbox}->pack_end($s->{_online_button},0,0,1);
        }
        if (RSSReader::Gtk2UI::Preferences->new()->online) {
                $s->{pix}->set_from_file(RSSReader::Gtk2UI::IconFactory->filename('online'));
                $s->{_online_button_tooltips}->set_tip( $s->{_online_button}, "Working Online, Click here to go offline", "" );
                $s->status_bar_remove($s->{_offline_mid}) if $s->{_offline_mid};
                delete $s->{_offline_mid} if $s->{_offline_mid};
        } else {
                $s->{pix}->set_from_file(RSSReader::Gtk2UI::IconFactory->filename('offline'));
                $s->{_online_button_tooltips}->set_tip( $s->{_online_button}, "Working Offline, Click here to go offline", "" );
                $s->{_offline_mid} = $s->status_bar_add("You are currently working offline") unless $s->{_offline_mid};
        }
	return 1;

}


sub _init_status_bar {
	my $s = shift;
	print STDERR "Initing status bar\n";
	$s->{_status_hbox} = Gtk2::HBox->new;
	$s->{_status_bar} = Gtk2::Statusbar->new();
	$s->{_status_bar}->show;
	$s->{_status_hbox}->pack_end($s->{_status_bar},1,1,1);
	$s->{_status_hbox}->show_all;
	$s->{_cont_id} = $s->{_status_bar}->get_context_id('Main window');	
	$s->{_widget}->set_statusbar_custom($s->{_status_hbox},$s->{_status_bar});
	return 1;
}

sub _init_menubar {
	my $s = shift;
	
	my @items = (
        [
                '/_File',
                undef,
                undef,
                undef,
                '<Branch>',
        ],
        [
                '/_File/Save',
                '<ctrl>S',
                sub { RSSReader::Gtk2UI::SaveConfiguration->run() },
                1,
                '<StockItem>',
                'gtk-save'
        ],
        [
                '/_File/Import',
                undef,
                undef,
                1,
                '<Branch>',
                'gtk-add'
        ],
        [
                '/_File/Export',
                undef,
                undef,
                1,
                '<Branch>',
                'gtk-add'
        ],
        [
                '/_File/Quit',
                '<ctrl>Q', 
                sub { RSSReader::Gtk2UI->new()->_exit },
                1,
                '<StockItem>',
                'gtk-quit'
        ],
	[	'/_Tools',
		undef,
		undef,
		1,
		'<Branch>',
		''
	],
	[	'/_Tools/Preferences',
		undef,
		sub { RSSReader::Gtk2UI::GeneralProperties->run },
		1,
		'<StockItem>',
		'gtk-preferences'
	],
	[	'/_Tools/New Feed',
		undef,
		sub { RSSReader::Gtk2UI::AddAFeedToCategory->run(category=>RSSReader::Category->new(id=>1)) },
		1,
		'<StockItem>',
		'gtk-add'
	],
	[	'/_Tools/New Category',
		undef,
		sub { RSSReader::Gtk2UI::AddACategory->run(category=>RSSReader::Category->new(id=>1)) },
		1,
		'<StockItem>',
		'gtk-add'
	],
	[	'/_Tools/Discover Feed',
		undef,
		sub {RSSReader::Gtk2UI::DiscoverFeed->run(category=>RSSReader::Category->new(id=>1))  },
		1,
		'<StockItem>',
		'gtk-add'
	],
	[	'/_Actions',
		undef,
		undef,
		1,
		'<Branch>',
		''
	],
	[	'/_Actions/Fetch all feeds',
		undef,
		sub { RSSReader::Gtk2UI->new->get_feeds(force=>1) },
		1,
		'<StockItem>',
		'gtk-refresh'
	],
	[	'/_Actions/Fetch current',
		undef,
		sub { $s->{_current_view}->do_fetch() },
		1,
		'<StockItem>',
		'gtk-refresh'
	],
	[	'/_Actions/Work online',
		undef,
		sub { RSSReader::Gtk2UI::Preferences->new->online(1) },
		1,
		'<StockItem>',
		''
	],
	[	'/_Actions/Work offline',
		undef,
		sub { RSSReader::Gtk2UI::Preferences->new->online(0) },
		1,
		'<StockItem>',
		''
	],
	);

	#Importers
	foreach my $imp (@{RSSReader::Importer->get_importers()}) {
		push @items, [
                		'/_File/Import/'.$imp->name,
				undef,
                		sub { RSSReader::Gtk2UI::ImporterDruid->run($imp) },
                		1,
                		'<StockItem>',
                		'gtk-add'
        		     ];
	}

	#Exporters
	foreach my $exp (@{RSSReader::Exporter->get_exporters()}) {
		push @items, [
                		'/_File/Export/'.$exp->name,
				undef,
                		sub { RSSReader::Gtk2UI::ExporterDruid->run($exp) },
                		1,
                		'<StockItem>',
                		'gtk-add'
        		     ];
	}

	#create menu
	$s->{_factory} = Gtk2::ItemFactory->new('Gtk2::MenuBar', '<main>', undef);
	$s->{_factory}->create_items('foo', @items);
	my $menubar = $s->{_factory}->get_widget('<main>');
	
	#add accelerators
	my @accels = (
       	 { key => 'S', mod => 'control-mask', func => sub { RSSReader::Gtk2UI::SaveConfiguration->run() } },
       	 { key => 'Q', mod => 'control-mask', func => sub { RSSReader::Gtk2UI->new->_exit() } },
	);
	my $accel_group = Gtk2::AccelGroup->new;
	foreach my $a (@accels) {
           $accel_group->connect($Gtk2::Gdk::Keysyms{$a->{key}}, $a->{mod}, 'visible', $a->{func});
	}
	$s->{_widget}->add_accel_group ($accel_group);

	$s->{_widget}->set_menus($menubar);

}

sub _init_toolbar {
	my $s = shift;
	$s->{_toolbar} = new Gtk2::Toolbar();
        $s->{_toolbar_save} = $s->{_toolbar}->append_item( 'Save', 'Save', 'Save',
							    Gtk2::Image->new_from_stock('gtk-save','large-toolbar'),
							    sub { RSSReader::Gtk2UI::SaveConfiguration->run() }
							    );
	$s->{_toolbar_fetch}= $s->{_toolbar}->append_item( 'Fetch current','Fetch Current','Fetch Current', 
							    Gtk2::Image->new_from_stock('gtk-refresh','large-toolbar'),
							    sub { $s->{_current_view}->do_fetch }
							    );
	$s->{_toolbar_prefs}= $s->{_toolbar}->append_item( 'View preferences','View preferences','View preferences', 
							    Gtk2::Image->new_from_stock('gtk-preferences','large-toolbar'),
							    sub { $s->{_current_view}->do_show_preferences }
							    );
	$s->{_toolbar_nex}  = $s->{_toolbar}->append_item( 'Next unread','Next unread','Next unread', 
							    Gtk2::Image->new_from_stock('gtk-go-down','large-toolbar'),
							    sub { $s->_next_unread }
							    );
	$s->{_toolbar}->show_all();
	$s->{_widget}->set_toolbar($s->{_toolbar});
	$s->{_toolbar}->set_style( 'icons' );

}

sub _exiting {
        my $s = shift;
        $s->log_debug("Main Window exiting");                                                                                                                                                           
        #hide to save view preferences
        $s->{_current_view}->hide() if $s->{_current_view};

        my $prefs = RSSReader::Gtk2UI::Preferences->new();

        my ($width, $heigth ) = $s->{_widget}->window->get_size;
        $prefs->set('main_window', 'width', $width);
        $prefs->set('main_window', 'heigth', $heigth);
        $prefs->set('main_window', 'gutter_pos', $s->{_pane}->get_position );
}


1;
