package RSSReader::Gtk2UI::Message::Differ;
use strict;
use Algorithm::Diff 'sdiff';
use Data::Dumper;

our @UNBALANCED_TAGS = qw(br hr p li base basefont meta link 
			  col colgroup frame input isindex area 
			  embed img bgsound marquee);


sub diff {
    my $c = shift;
    my ($new, $old) = @_;
    print STDERR "$new\n\n$old\n";
    my $result = &html_diff( $old, $new);
    print STDERR Dumper($result);
    my $buffer = "";
    foreach my $r ( @$result ) {
        my $o = $r->[1];
        my $n = $r->[2];
        $r->[0] = 'u' if ($o eq $n);
	next unless $r->[0];
        if ( $r->[0] eq 'u' ) {
                $buffer.=$n;
        } elsif ( $r->[0] eq 'c' ) {
                $buffer .= "<font color=green>$n<\/font><span style=\"text-decoration: line-through; color: red;\">$o<\/span>";
        } elsif (  $r->[0] eq '+' || $r->[0] eq '-') {
		if (! $o) {
                  $buffer .= "<font color=green>$n<\/font>";
		}
		elsif ( ! $n) {
		  $buffer .= "<span style=\"text-decoration: line-through; color: red;\">$o<\/span>";

		}
		else {
		  $buffer .= "<font color=green>$n<\/font> <span style=\"text-decoration: line-through; color: red;\">$o<\/span>";
		}
	}
   }

   print STDERR $buffer."\n";
   
   return $buffer;

}

sub member {
    my ($item, @list) = @_;

    foreach my $example (@list) {
	return 1 if $example eq $item;
    }
    return 0;
}

sub html_diff {
    my ($left, $right) = @_;

    # Split the two texts into words and tags.
    my (@leftchks) = $left =~ m/(<[^>]*>\s*|[^<]+)/gm;
    my (@rightchks) = $right =~ m/(<[^>]*>\s*|[^<]+)/gm;
    
    @leftchks = map { $_ =~ /^<[^>]*>$/ ? $_ : ($_ =~ m/(\S+\s*)/gm) }
                    @leftchks;
    @rightchks = map { $_ =~ /^<[^>]*>$/ ? $_ : ($_ =~ m/(\S+\s*)/gm) } 
                     @rightchks;

    # Remove blanks; maybe the above regexes could handle this?
    @leftchks = grep { $_ ne '' } @leftchks;
    @rightchks = grep { $_ ne '' } @rightchks;

    my $tagstack = [];
    my $smear_tags = sub {
	if ($_ =~ /^<.*>/) {
	    if ($_ =~ m|^</|) {
		my ($tag) = m|^</\s*([^ \t\n\r>]*)|;
		# If we found the closer for the tag on top 
		# of the stack, pop it off.
		if ($$tagstack[-1] eq $tag) {
                    my $stacktag = pop @$tagstack;
                }
		return [$_, $tag];
	    } else {
		my ($tag) = m|^<\s*([^\s>]*)|;
		$tag = lc $tag;
		if (member($tag, @UNBALANCED_TAGS) || $tag =~ m#/\s*>$#)
		{	                # (tags without correspond closer tags)
		    return [$_, $tag];
		} else {
		    push @$tagstack, $tag;
		}
		return [$_, $_];
	    }
	} else {
	    my $result = [$_, (join "!!!", (@$tagstack, $_)) ];
	    return $result;
	}
    };

    $tagstack = [];
    @leftchks = map { &$smear_tags } @leftchks;

    $tagstack = [];
    @rightchks = map { &$smear_tags } @rightchks;

    my $chunks = sdiff(\@leftchks, \@rightchks,
		      sub { $_ = elem_cmprsn(shift); $_ =~ s/\s+$/ /g; $_ });

    my $lastsignal = '';
    my ($lbuf, $rbuf);
    my @result;
    my $ch;
    foreach $ch (@$chunks) {
	my ($signal, $left, $right) = @$ch;
	if ($signal eq 'u' && $lastsignal ne 'u') {
	    push @result, [$lastsignal, $lbuf, $rbuf];
	    $lbuf = "";
	    $rbuf = "";
	} elsif ($signal ne 'u' && $lastsignal eq 'u') {
	    push @result, [$lastsignal, $lbuf, $rbuf];
	    $lbuf = "";
	    $rbuf = "";
	}
	$lbuf .= elem_mkp($left) || '';
	$rbuf .= elem_mkp($right) || '';
	$lastsignal = $signal;
    }
    push @result, [$lastsignal, $lbuf, $rbuf];
    return \@result;
}

sub elem_mkp {
    my ($e) = @_;
    return undef unless ref $e eq 'ARRAY';
    my ($mkp, $cmp) = @$e;
    return $mkp;
}

sub elem_cmprsn {
    my ($e) = @_;
    return undef unless ref $e eq 'ARRAY';
    my ($mkp, $cmp) = @$e;
    return $cmp;
}

1;
