package RSSReader::Gtk2UI::MessageList;
use RSSReader::Gtk2UI::IconFactory;
use RSSReader::Gtk2UI::MainWindow;
use strict;
use Glib qw(TRUE FALSE);
use HTTP::Date;

sub new {
	my $c = shift;
	my %a = @_;
	my $s = bless {}, $c;
	my $scrolled_win = new Gtk2::ScrolledWindow( undef, undef );
	
	#Create a scrolled window for the list;
	$scrolled_win->set_policy( 'automatic', 'automatic' );
	$scrolled_win->show();
	$s->{_widget} = $scrolled_win;
	
	$s->{_list_store} =  Gtk2::TreeStore->new (qw/
							Glib::String
							Glib::String
							Glib::String
							Glib::String 
							Glib::Boolean
							Glib::String 
							Glib::Boolean
							Glib::String 
						/);

    # proper sorting for the date column
    $s->{_list_store}->set_sort_func(5, sub {
        my ($model, $itera, $iterb, $data) = @_;
        my $timea = str2time($model->get_value($itera, 5));
        my $timeb = str2time($model->get_value($iterb, 5));

        return $timea <=> $timeb;
    }, undef);


	$s->{_list_view} = Gtk2::TreeView->new($s->{_list_store});
	$s->{_list_view}->set_rules_hint(1);
	$s->{_list_view}->set_headers_visible(1);

	# KEEP
	my $renderer_toggle = Gtk2::CellRendererToggle->new;
	my $column_tokeep   = Gtk2::TreeViewColumn->new_with_attributes ("K",
                                                          	$renderer_toggle,
                                                          	active => 4);
 	$s->{_list_view}->insert_column($column_tokeep,0);	
	$renderer_toggle->signal_connect('toggled', sub { $_[2]->_to_keep_w_toggled(cell=>$_[0], path=>$_[1]) }, $s);

	#feed name;	
	my $renderer = Gtk2::CellRendererText->new;
	$s->{_list_view}->insert_column_with_data_func(
			1,
			"Feed",
			$renderer,
			sub {
				my ($treecol, $cell, $model, $iter, $col_num) = @_;
				my $feed_name = $model->get ($iter, 1);

				if ( $model->get($iter, 6) ) { $cell->set( text => $feed_name, weight=>0 ) }
				else 			     { $cell->set( text => $feed_name, weight=>60000) }
			},
			undef
	);
	
	#image date 
	my $renderer_date = Gtk2::CellRendererText->new;
	$s->{_list_view}->insert_column_with_data_func(
			2,
			"Date",
			$renderer_date,
			sub {
				my ($treecol, $cell, $model, $iter, $col_num) = @_;
				my $date = $model->get ($iter, 5);

				if ( $model->get ($iter, 6) ) 	{ $cell->set( text => $date, weight=>0      ) }
				else				{ $cell->set( text => $date, weight=>60000  ) }
			},
			undef
	);
	
	#message title 
	$s->{_list_view}->insert_column_with_data_func(
			3,
			"Title",
			$renderer,
			sub {
				my ($treecol, $cell, $model, $iter, $col_num) = @_;
				my $title = $model->get($iter, 3);
				if ( $model->get ($iter, 6) ) 	{ $cell->set( text => $title, weight=>0      ) }
				else				{ $cell->set( text => $title, weight=>60000  ) }
			},
			undef
	);

	#message category 
	$s->{_list_view}->insert_column_with_data_func(
			4,
			"Category",
			$renderer,
			sub {
				my ($treecol, $cell, $model, $iter, $col_num) = @_;
				my $cat = $model->get ($iter, 7);
				if ( $model->get ($iter, 6) ) 	{ $cell->set( text => $cat, weight=>0      ) }
				else				{ $cell->set( text => $cat, weight=>60000  ) }
			},
			undef
	);
	
	$s->{_list_view}->set_headers_clickable(1);
	$s->{_list_store}->signal_connect('sort-column-changed', sub { $_[1]->_sort_changed() }, $s );
	$s->{_list_store}->set_sort_column_id(5, 'descending');
	
	$s->{_widget}->add($s->{_list_view});
	my ( $column_tokeep, $col_name, $col_date, $col_title, $col_category) =
		($s->{_list_view}->get_column(0), $s->{_list_view}->get_column(1), 
		 $s->{_list_view}->get_column(2), $s->{_list_view}->get_column(3), $s->{_list_view}->get_column(4));
	$col_name->{_sorts} = 1;
	$col_date->{_sorts} = 5;
	$col_title->{_sorts} = 3;
	$column_tokeep->{_sorts} = 4;
	$col_category->{_sorts} = 7;

	my $k = 0;
	while (my $col = $s->{_list_view}->get_column($k) ) {
		$col->set_resizable(1);
		$col->{_position} = $k;
		$col->set_sizing('fixed');
		$col->set_fixed_width('25');
		$col->signal_connect('clicked', sub { $_[1]->_change_sort(@_) }, $s );
		$k++;
	}
	$column_tokeep->set_sizing('autosize');
	$column_tokeep->set_sizing('fixed');
	$column_tokeep->set_fixed_width('25');
	$column_tokeep->set_clickable(0);
	$column_tokeep->set_resizable(0);
	my $image = Gtk2::Image->new_from_stock('syndigator-tokeep','button');
	$image->show;
	$column_tokeep->set_widget($image);
	$column_tokeep->get_widget->show();
	$column_tokeep->set_alignment(0.5);
	
	my $selection = $s->{_list_view}->get_selection();
	$selection->set_mode('extended');
	$selection->signal_connect('changed', sub { $_[1]->_selection_changed($_[0]) }, $s );

	#right click managing...
	$s->{_list_view}->signal_connect('button-press-event', sub { my ($w, $e, $s) = @_; return $s->_button_press(widget=>$w, event=>$e) },$s);

	$s->{_prefs} = RSSReader::Gtk2UI::Preferences->new();
	$s->{_prefs}->notify_me(event=>"show_category_col_changed", caller=>$s, call=>'_setup_category_column');
	$s->_setup_category_column();

	return $s;
}

sub _setup_category_column {
	my $s = shift;
	my $col = $s->{_list_view}->get_column(4);
	if ( $s->{_prefs}->option("show_category_col") ) {
		$col->set_visible(1);
	} else {
		$col->set_visible(0);
	}
}

sub show{
	my $s = shift;
	return 1;
}

sub _selection_changed {
    my $s = shift;
    my $selection = shift;

    return if ($selection->count_selected_rows() >1);

    my $path = $selection->get_selected_rows();
    return unless $path;
    my $iter = $s->{_list_store}->get_iter($path);
    my $msg = $s->_get_message_for_iter($iter);

    $s->log_debug("Message ".$msg->title." was selected");
    
    $s->{_list_view} -> grab_focus();

    $s->notify("message_selected", message => $msg);
    RSSReader::Gtk2UI::MainWindow->new->notify("message_selected", message => $msg);
}

sub listed_messages {
    my $s = shift;
    my @to_return = ();

    my $iter = $s->{_list_store}->get_iter_first ();
    while ($iter) {
	my $msg = $s->_get_message_for_iter($iter);
	push @to_return, $msg;
	$iter = $s->{_list_store}->iter_next($iter);	
    }

    return wantarray ? @to_return : \@to_return; 
}

sub _foreach {
    my $s = shift;
    my $action = shift;

    my $iter = $s->{_list_store}->get_iter_first ();
    while ($iter) {
	my $msg = $s->_get_message_for_iter($iter);
   	if ($action eq 'mark_read') {
		$msg->mark_read;
	}
	elsif ($action eq 'mark_unread') {
		$msg->mark_unread;
	}
	elsif ($action eq 'mark_to_keep') {
		$msg->mark_to_keep;
	}
	elsif ($action eq 'mark_not_to_keep') {
		$msg->mark_to_remove;
	}
	$iter = $s->{_list_store}->iter_next($iter);	
    }
}

sub select_next_unread {
    my $s = shift;

    $s->log_debug("Should get next unread");
    
    my $selection = $s->{_list_view}->get_selection();
    my $iter;
    my @selected = $selection->get_selected_rows;
    
    if    (scalar(@selected) > 1)    { $iter = $s->{_list_store}->get_iter_first		    }
    elsif (scalar(@selected) == 1)   { $iter = $s->{_list_store}->get_iter($selected[0])	    }
    else 	      		     { $iter = $s->{_list_store}->get_iter_first  		    }
    my $_orig_msg = $s->_get_message_for_iter($iter);
    while ($iter) {
    	my $m = $s->_get_message_for_iter($iter);
	if (! $m->read) {
		$selection->unselect_all();
		$s->log_debug("selected iter");
		my $p = $s->{_list_store}->get_path($iter);
		$s->{_list_view}->set_cursor($p, undef, FALSE);
		$s->log_debug("cursor set");
		$s->{_list_view}->scroll_to_cell($p, undef, 0,1,1);
		$s->log_debug("scrolled to cell");
		$selection->select_iter($iter);
		last;
	}
	
	$iter = $s->{_list_store}->iter_next($iter) || $s->{_list_store}->get_iter_first ;
    }
}

sub _to_keep_w_toggled {
    my $s = shift;
    my %a = @_;

    my $path = Gtk2::TreePath->new($a{path});
    my $iter = $s->{_list_store}->get_iter($path);
    my ($feed_id, $message_id) = $s->{_list_store}->get($iter,0,2);

    RSSReader::Feed::Message->new(feed=>RSSReader::Feed->new(id=>$feed_id), id=>$message_id)->toggle_keep;	

    return 1;

}

sub _message_changed {
    my $s = shift;
    my %a = @_;

    my $iter = $s->_find_iter_for_message($a{where});
    my $msg = $a{where};
    return unless $iter;
    $s->{_list_store}->set($iter, 0, $msg->get_feed->id,
                                  1, $msg->get_feed->label,
                                  2, $msg->id,
                                  3, $msg->title,
                                  4, $msg->to_keep,
                                  5, $msg->date,
				  6, $msg->read,
				  7, $msg->category);
    return 1;

}
sub _find_iter_for_message {
    my $s   = shift;
    my $msg = shift;

    my $iter = $s->{_list_store}->get_iter_first ();
    while ($iter) {
	my ($feed_id, $message_id) = $s->{_list_store}->get($iter,0,2);
	if (($feed_id eq $msg->get_feed->id) && ($message_id eq $msg->id)) {
		return $iter;
	}
	$iter = $s->{_list_store}->iter_next($iter);	
    }

    return $iter;
}

sub _get_message_for_iter {
    my $s = shift;
    my $iter = shift;

    my ($feed_id, $message_id) = $s->{_list_store}->get($iter,0,2);
    return RSSReader::Feed::Message->new( feed => RSSReader::Feed->new(id=>$feed_id), id => $message_id);
}

sub _sort {
    my $s = shift;
    my ($sort_column, $sort_type) = $s->{_list_store}->get_sort_column_id;
    $s->{_list_store}->set_sort_column_id($sort_column,$sort_type);
}

sub _change_sort {
    my $s = shift;
    my $view_column = shift;

    
    my $store_sort_column = $view_column->{_sorts};

    my ($sort_column, $sort_type) = $s->{_list_store}->get_sort_column_id; 

    if ( $sort_column == $store_sort_column) {
	if ( $sort_type eq 'ascending' ) {  $s->{_list_store}->set_sort_column_id($store_sort_column,'descending')  }
	else {	$s->{_list_store}->set_sort_column_id($store_sort_column,'ascending')  }
	return 1;
    }

    $s->{_list_store}->set_sort_column_id($store_sort_column,'descending');
    return 1;
    
}

sub _sort_changed {
    my $s = shift;
    my ($sort_column, $sort_type) = $s->{_list_store}->get_sort_column_id;
    $s->log_debug("Tree is now ordered by: $sort_column, $sort_type");

    my $view_column;
    if ($sort_column == 1) { $view_column = 1; }  #FEED LABEL
    if ($sort_column == 5) { $view_column = 2; }  #Date
    if ($sort_column == 3) { $view_column = 3; }  #title
    if ($sort_column == 4) { $view_column = 0; }  #to keep
    if ($sort_column == 7) { $view_column = 4; }  #category

    if ($s->{_current_sort_view_column} && ($s->{_current_sort_view_column} == $view_column) ) {
    	my $vcol = $s->{_list_view}->get_column($view_column);
	$vcol->set_sort_order($sort_type)
    }

    else {
    	if (exists($s->{_current_sort_view_column}) && defined($s->{_current_sort_view_column}) ) {
		my $wrong_col = $s->{_list_view}->get_column($s->{_current_sort_view_column});
		$wrong_col->set_sort_indicator(0);
    	}
	my $vcol = $s->{_list_view}->get_column($view_column);	
	$vcol->set_sort_indicator(1);
	$vcol->set_sort_order($sort_type);
	$s->{_current_sort_view_column} = $view_column;
    }

    return 1;
    
}

sub widget { return shift->{_widget} }


sub _foreach_selected {
    my $s = shift;
    my $action = shift;

    my $selection = $s->{_list_view}->get_selection();
    my @paths = $selection->get_selected_rows();	
    foreach my $p ( @paths ) {
		my $iter = $s->{_list_store}->get_iter($p);
		my $msg = $s->_get_message_for_iter($iter);
    		if ($action eq 'mark_read') {
			$msg->mark_read;
		}
		elsif ($action eq 'mark_unread') {
			$msg->mark_unread;
		}
		elsif ($action eq 'mark_to_keep') {
			$msg->mark_to_keep;
		}
		elsif ($action eq 'mark_not_to_keep') {
			$msg->mark_to_remove;
		}
    }
}
sub _delete_message {
    my $s = shift;
    my %a = @_;
    $a{message} = $a{where} unless $a{message};
    return unless $a{message};
                                                                                                                                              
    $s->log_debug("A message was deleted, should remove");
    my $iter = $s->_find_iter_for_message($a{message});
    return unless $iter;

    $s->{_list_store}->remove($iter);
    $s->_sort;
}


sub _popup {
    my $s = shift;
    my %a = @_;

    my @items = (
        [
                '/Mark read',
                undef,
                sub { $s->_foreach_selected('mark_read') },
                1,
                '<StockItem>',
                ''
        ],
        [
                '/Mark unread',
                undef,
                sub { $s->_foreach_selected('mark_unread')},
                1,
                '<StockItem>',
                ''
        ],
        [
                '/Mark to keep',
                undef,
                sub { $s->_foreach_selected('mark_to_keep')},
                1,
                '<StockItem>',
                ''
        ],
        [
                '/Mark not to keep',
                undef,
                sub { $s->_foreach_selected('mark_not_to_keep')},
                1,
                '<StockItem>',
                ''
        ],
        [
                '/separator',
                undef,
                undef,
                1,
                '<Separator>',
                ''
        ],
        [
                '/Mark all read',
                undef,
                sub { $s->_foreach('mark_read') },
                1,
                '<StockItem>',
                ''
        ],
        [
                '/Mark all unread',
                undef,
                sub { $s->_foreach('mark_unread')},
                1,
                '<StockItem>',
                ''
        ],
        [
                '/Mark all to keep',
                undef,
                sub { $s->_foreach('mark_to_keep')},
                1,
                '<StockItem>',
                ''
        ],
        [
                '/Mark all not to keep',
                undef,
                sub { $s->_foreach('mark_not_to_keep')},
                1,
                '<StockItem>',
                ''
        ],

    );
    
    my $factory = Gtk2::ItemFactory->new('Gtk2::Menu', '<messagelistpopup>', undef);

    $factory->create_items(1, @items);

    if (scalar(@{$a{messages}}) == 1 ) {
    	$s->log_debug("one message selected, should tune popup");
	my $m = $a{messages}->[0];
	if ($m->read) { 
		$factory->get_item('<messagelistpopup>/Mark read')->set_sensitive(0);
	}
	else {
		$factory->get_item('<messagelistpopup>/Mark unread')->set_sensitive(0);
	}
	if ($m->to_keep) {
		$factory->get_item('<messagelistpopup>/Mark to keep')->set_sensitive(0);
	}
	else {
		$factory->get_item('<messagelistpopup>/Mark not to keep')->set_sensitive(0);
	}
    }
    my $menu = $factory->get_widget('<messagelistpopup>');
    $menu->popup(undef, undef, undef,undef, 0, Gtk2->get_current_event_time());
}

sub cleanup {
    my $s = shift;
    my %a = @_;

    foreach my $m ( @{$s->{_to_remove}} ) {
    	$s->log_debug("Cleaning upmessage ".$m->title);
	$s->_delete_message(message=>$m);
    }

    $s->{_to_remove} = [];
}

sub add_msg {
    my $s = shift;
    my $msg = shift;

    $msg->notify_me(event=>'keep_toggled',   call=>'_message_changed', caller=>$s);
    $msg->notify_me(event=>'mark_read',      call=>'_message_changed', caller=>$s);
    $msg->notify_me(event=>'mark_unread',    call=>'_message_changed', caller=>$s);
    $msg->notify_me(event=>'message_deleted',call=>'_delete_message',  caller=>$s);
    my $iter = $s->{_list_store}->append(undef);
    $s->{_list_store}->set($iter, 0, $msg->get_feed->id, 
    				  1, $msg->get_feed->label,
				  2, $msg->id,
				  3, $msg->title(escaped=>1),
				  4, $msg->to_keep,
				  5, $msg->date,
				  6, $msg->read,
				  7, $msg->category
			  );
    $s->_sort;

}

sub _new_message {
    my $s = shift;
    my %a = @_;

    my $iter = $s->_find_iter_for_message($a{message});
    if ($iter) {
    	#FIXME se estiver para remover tira!!
	return;
    };
    $s->add_msg($a{message});
}

sub _button_press {
    my $s = shift;
    my %a = @_;

    my $selection = $s->{_list_view}->get_selection;
    my @selected = $selection->get_selected_rows();
    $s->log_debug("button pressed:  ".$a{event}->button." as ".$a{event}->type);

    if ($a{event}->button == 3) {
	print "its a rigth click :)\n";

	if ($selection->count_selected_rows() <= 1 ) {
		print "We are on single\n";
		my ($p) = $s->{_list_view}->get_path_at_pos($a{event}->x, $a{event}->y);
		
		my $message_clicked;
		my $message_selected;
		
		$message_clicked  = $s->_get_message_for_iter( $s->{_list_store}->get_iter($p) ) if $p;
		$message_selected = $s->_get_message_for_iter( $s->{_list_store}->get_iter($selected[0]) ) if defined($selected[0]);

		

		if ( (!$message_clicked) || 
		     (!$message_selected)|| 
                     ($message_clicked->id ne $message_selected->id ) ) {
			$s->log_debug("Should change message");
			$selection->unselect_all;
			$selection->select_path($p);	
		}
	}

	print "Should pop up on selected";
	my @m = ();
	foreach my $p ( @selected ) {
			next unless $p;
	                my $iter = $s->{_list_store}->get_iter($p);
		        my $msg = $s->_get_message_for_iter($iter);
			push @m, $msg if $msg;
	}
					
	$s->_popup(messages => \@m);

	return 1;

    }
    elsif (($a{event}->button == 1) && ($a{event}->type =~ /\dbutton-press/ )) {
	#its a double ckick

	#should check if is inside the list
	#my $path = $s->{_list_view}->get_path_at_pos ($x, $y)

	$s->log_debug("Should show message url");
	my $message_selected = $s->_get_message_for_iter( $s->{_list_store}->get_iter($selected[0]) ) if defined($selected[0]);
	my $url = $message_selected->url; 
	RSSReader::Gtk2UI->launch_url($url);
    }

    return undef;
}

1;
