package RSSReader::Gtk2UI::Notifiers::PopUpWindow;
use RSSReader;
use base qw(RSSReader::Gtk2UI::Notifiers RSSReader::Logger);
use strict;

our $_notifier_pop_up_window;
our $_failed_load = undef;

sub new { 
  my $c = shift;

  return $_notifier_pop_up_window if $_notifier_pop_up_window;
 
  my $s = { };
  bless $s, $c;

  RSSReader->new()->notify_me(event =>'new_message', caller=>$s, call=>'_new_message');

  $_notifier_pop_up_window = $s;
  return $_notifier_pop_up_window;

}

sub is_active {
   my $c = shift;	
   return $_failed_load ? 0 : 1 ;
}

sub fail_reason {
   my $c = shift;
   return $_failed_load;
}

sub id { return "notifier_popupwindow"; }

sub name { return "Pop Up Window"; }

sub version { return '0.01' }

sub config_options { 
     my $s = shift;
     return [
	{ label   => "Window timeout",
	  name    => 'window_timeout',
	  default => 10,
	  type    => 'spin integer',
	}
     ];
}

sub _new_message { 
    my $s = shift;
    my %a = @_;
    $s->log_debug("New message as arrived : ".$a{message}->title());

    return unless $a{message}->get_feed->notify_new_messages;
    return unless $s->is_enabled;

    $s->_notify_feed($a{message}->get_feed);
    return 1;
}

sub _destroy_window {
    my $s = shift;

    $s->log_debug("I should be detroyed");
    return 1 unless exists($s->{_window});
    if ($s->{_window}->{tout}) {  Glib::Source->remove( $s->{_window}->{tout} ) }
    $s->{_window}->{widget}->destroy();
    delete $s->{_window};
    return 1;
}

sub _notify_feed {
    my $s    = shift;
    my $feed = shift; 
    return unless $s->is_enabled;
    return unless $s->is_active;
    $s->log_debug("I am enabled");

    my $w_timeout= ($s->get_option("window_timeout")*1000) || 10000;

    unless (exists $s->{_window}) {
    	$s->log_debug("Window is not displayed");
	my $w = new Gnome2::App( 'toplevel',"Syndigator notifier" );;
	$w->set_decorated(0);
	$s->{_window}->{widget} = $w;
	my $table = new Gtk2::Table(2,3,0);
	$table->show_all();
	my $l = new Gtk2::Label('The following feeds have new messages:');
	$l->show();
	$table->attach($l ,0,2,0,1,['fill','shrink'],[], 1,1);

	my $frame = new Gtk2::Frame;
	$table->attach($frame, ,0,2,1,2,['fill','shrink'],[], 1,1);
	my $vbox = new Gtk2::VBox();
	$vbox->show();
	$s->{_window}->{_vbox} = $vbox;

	$frame->add($vbox);
	$frame->show();

	my $ok = Gtk2::Button->new_from_stock('gtk-ok');;
	$ok->show();
	$table->attach($ok, ,0,2,2,3, [],[], 1,1);
	
	$w->set_contents($table);
	$w->show();

	$ok->signal_connect_after('clicked', sub { $s->_destroy_window } );
        $s->log_debug("Timeout: [$w_timeout]");

	$s->{_window}->{tout} = Glib::Timeout->add($w_timeout, sub { $s->_destroy_window });
    }
    if ( exists $s->{_window}->{notifing_feeds}->{$feed->id} ) {
	$s->log_debug("I was already notifying this feed");
	return;
    }

    if ($s->{_window}->{tout}) {  Glib::Source->remove( $s->{_window}->{tout} ) }
    $s->{_window}->{tout} = Glib::Timeout->add($w_timeout, sub { $s->_destroy_window });

    $s->log_debug("Should notify this feed");
    $s->{_window}->{notifing_feeds}->{$feed->id} = 1;
    my $label = new Gtk2::Label($feed->label);
    $label->show;
    $s->{_window}->{_vbox}->pack_start($label, 0,0,1);
}

sub description {
    return "This notifier pops a window every time a feed (marked for notification) receives new messages";
}


RSSReader::Gtk2UI::Notifiers->register( id => __PACKAGE__->id, package => __PACKAGE__ );
1;
