package RSSReader::Importer::Evolution;

use strict;
use RSSReader::Category;
use RSSReader::Feed;
use RSSReader::Importer;
use Data::Dumper;
use base qw( RSSReader::Logger RSSReader::Importer );


sub new {
  my $c = shift;
  my %a = @_;
  my $evo_dir = $a{dir} || $ENV{HOME}.'/evolution/';

  return bless { evo_dir => $evo_dir}, $c
}

sub fetch {
  my $s = shift;
  my %a = @_;

  my @feeds;

  $s->log_spam("Importing Evolution RDFs...");

  my $evo_dir = $a{dir};
  unless (-d $evo_dir) {
    $s->log_warn("$evo_dir does not exist");
    $@ = "Given Evolution dir does not exist";
    return undef;
  }

  unless (open(FH, '<', $evo_dir.'RDF-urls.txt')) {
    $s->log_warn("Can't open evolution RDF-urls.txt: $!...");
    $@ = "Could not find evolution RDF storage file";
    return undef;
  }
  my $counter = 0;
  local $_;
  while (<FH>) {              
    $counter++;
    if (/^(.*),(.*)$/) {
      push @feeds, { 
                     label          => $2,
                     url            => $1,
		     parent	    => undef,
		     id 	    => $counter
                   };
      next;
    }
  }
  close(FH);
  $s->log_spam("...Fetched\n");
  my %to_return = (
  		    feeds      => \@feeds,
		    categories => []
  		  );

  return wantarray ? %to_return : \%to_return;
}


sub id { return "importer_evolution"; }

sub name { return "Evolution"; }

sub config_options {
  my $s=shift; 
  return [{
            default => $s->{evo_dir}, 
            name    => 'dir', 
            desc    => 'Evolution home directory', 
            type    => 'text'
         }];
}

sub description {
	return 'Import your Evolution Mail Reader RSS Feeds';
}

RSSReader::Importer->register( id => __PACKAGE__->id, package => __PACKAGE__ );
1;
