package RSSReader::Notifier;

use strict;
use Data::Dumper;
use base qw( RSSReader::Logger );

sub notify_me {
  my $s = shift;
  my %a = @_;

  $s->log_spam("$a{caller} wants to be notified of $a{event}");

  foreach my $k ( keys %{$s->{_events_to_notify}->{$a{event}}} ) {
	my $n = $s->{_events_to_notify}->{$a{event}}->{$k};
	if ( $n->{package} eq $a{caller} && $n->{call} eq $a{call} ) {
		$s->log_spam("event $a{event} is already subscribed by you ($n->{package}) to call $n->{call}");
		return undef;
	}
  }

  my $id = int(rand(100))+1;
  while ( exists $s->{_events_to_notify}->{$a{event}}->{$id} ) {
	$id++;
  }

  $s->{_events_to_notify}->{$a{event}}->{$id} = { package => $a{caller}, call=>$a{call} };
  return $id;
}

sub forget_notify_me {
  my $s = shift;
  my %a = @_;
  
  if ( $a{id} ) {
	delete $s->{_events_to_notify}->{$a{event}}->{$a{id}};
	return 1;
  }  

  foreach my $k ( keys %{$s->{_events_to_notify}->{$a{event}}} ) {
	my $n = $s->{_events_to_notify}->{$a{event}}->{$k};
	if ( $n->{package} eq $a{caller} && $n->{call} eq $a{call} ) {
		delete $s->{_events_to_notify}->{$a{event}}->{$k};	
	}
  }
}

sub notify {
  my $s     = shift;
  my $event = shift;
  $s->log_debug("I have a '".$event."' event");
  foreach my $k ( keys %{$s->{_events_to_notify}->{$event}} ) {
	my $to_call = $s->{_events_to_notify}->{$event}->{$k};
  	$s->log_debug("Notifying ".(ref($to_call) || $to_call));
	my $obj = $to_call->{package};
	my $call = $to_call->{call};
	eval { 
	 $obj->$call(where=>$s, @_) 
	};	
	if ($@) {
		$s->log_error("ERROR, something very wrong : $@");
	}
  }
}
	
1;
