package RSSReader::Parser::Atom;
use Data::Dumper;
use XML::Simple;
use base qw( RSSReader::Logger RSSReader::Parser );
use strict;
                                                                                                                       
                                                                                                                       
sub new {
  my $c = shift;
  my %a = @_;
                                                                                                                       
  return bless { }, $c
}

sub parse {
	my $c = shift;
	my $data = shift;
	my $ref = undef;
	my $could_parse=1;
                                                                                                                       
	eval {
	     my $xs = new XML::Simple(ForceArray=>['entry', 'link'],KeyAttr => { entry => "+id" });
	     $ref = $xs->XMLin($data);
	};
	if ($@ || (!$ref) ) {
	        $c->log_error("Could not parse this xml: ".Dumper($ref)."$@");
	        $could_parse=0;
		return (0,0,undef); 
	}

	if ($ref->{xmlns} !~ /http:\/\/purl.org\/atom\/ns/) {
		print $ref->{xmllns};
		return (0,0,undef); 
	}

	print STDERR Dumper($ref);

	my $feed = { items => [ ] };

	$feed->{title}        = $ref->{title}->{content};
	$feed->{description}  = $ref->{tagline}->{content};
	$feed->{version}      = "Atom ".$ref->{version};

	foreach my $e ( keys %{$ref->{entry}} ) {
		my $item = {};

		$item->{raw} = $ref->{entry}->{$e};

		$ref = $c->_convert_item($ref);	

		my $entry   = $ref->{entry}->{$e};
		my $title = "";
		print STDERR "******* REF ".ref($entry->{title})."\n";
		if ( $entry->{title} && ref($entry->{title}) ne 'HASH') {
			$title = $entry->{title};
		} elsif ( $entry->{title} && ref($entry->{title}) eq 'HASH' && $entry->{title}->{content} ) {
			$title = $entry->{title}->{content};
		}
		my $date    = $entry->{issued} || $entry->{created};
		my $content = $entry->{content}->{div}->{content} || $entry->{content}->{content};
		my $desc    = $entry->{summary}->{div}->{content} || $entry->{summary}->{content};
		my $author  = $entry->{author}->{name};
		my $link    = "";
		foreach my $l ( @{$entry->{link}} ) {
			if (lc($l->{rel}) eq 'alternate') {
				$link = $l->{href};
			}
		}

		if ( $author && $entry->{author}->{mail} ) {
			$author .= '<a href="mail_to:'.$entry->{author}->{mail}.'">'.$author.'</a>';
		} elsif ( $entry->{author}->{mail} ) {
			$author .= $entry->{author}->{mail};
		}
	
	
		$item->{title}       = $title;
		$item->{date}        = $c->_parse_date($date);
		$item->{url}         = $link;
		$item->{description} = $desc;
		$item->{fetch_time}  = time();
		$item->{content}     = $content;
		$item->{author}      = $author;
		$item->{id}          = $e;
		
		push @{$feed->{items}}, $item;
	}
	return (1,$could_parse,$feed); 
}

sub id { return "parser_atom"; }
                                                                                                                       
sub name { return "Atom Parser"; }
                                                                                                                       
RSSReader::Parser->register( id => __PACKAGE__->id, package => __PACKAGE__ );
1;
