package RSSReader::Parser::RSS;
##
# PARSES RSS/RDF Feeds

use strict;
use XML::Parser;
use Data::Dumper;
use XML::RSS;
use base qw( RSSReader::Logger RSSReader::Parser );


sub new {
  my $c = shift;
  my %a = @_;

  return bless { }, $c
}

sub parse {
   my $c = shift;
   my $data = shift;
   my $rss_o = undef;
   my $could_parse=1;

   eval {
     $rss_o = new XML::RSS();
     $rss_o->parse($data);
   };
   if ($@) {
	$c->log_warn("Could not parse this xml: ".Dumper($rss_o)."$@");
	$could_parse=0;
   }
   elsif ( !$rss_o->{version} ){
	$could_parse=0;
   }
   else {
  	my $fetch_time = time();
  	my $fetch_offset = scalar @{$rss_o->{'items'}};
	$rss_o->{title} = $rss_o->{channel}->{title};

  	foreach my $item (@{$rss_o->{'items'}}) {
        	#add fetch time to each message
	        $item->{fetch_time} = $fetch_time;
	        #avoid fetch_time clashing
	        $fetch_offset--;
        	$item = $c->_convert_item($item);
        	$item->{raw}  = $item;
        	$item->{url}  = $item->{link};
        	$item->{date} = $item->{dc}->{date} || $item->{pubDate};
        	$item->{date} = $c->_parse_date($item->{date}, $fetch_time + $fetch_offset);
		$item->{content} = $item->{'http://purl.org/rss/1.0/modules/content/'}->{'encoded'}
                	if exists($item->{'http://purl.org/rss/1.0/modules/content/'}->{'encoded'});
        	$item->{content} = $item->{'content:encoded'}
                	if exists($item->{'content:encoded'});
		$item->{author} |= $item->{info}->{dc}->{creator};
	}

   }

   $c->log_spam(Dumper($rss_o));

   #return ($is_mine, $could_parse, $parsed_data);
   return(1,$could_parse,$rss_o);
}

sub id { return "parser_rss"; }

sub name { return "RSS Parser"; }

RSSReader::Parser->register( id => __PACKAGE__->id, package => __PACKAGE__ );
1;
