package RSSReader::UI::Preferences;
use Data::Dumper;
use File::Path;
use XML::Simple;
use Storable;
use base qw(RSSReader::Logger RSSReader::Notifier);

our $_me_pref   = bless {}, __PACKAGE__;
our $PREFS_FILE = ( $ENV{SYNDIGATOR_DEBUG} ? 'preferences.xml' : 'preferences.sto' );

sub new {
  return $_me_pref;
}

sub get_current { return $current };

sub get {
  my $c    = shift;
  my $id   = shift;
  my $name = shift;

  die "Not enough params" unless ($id && $name);

  return $current->{stash}->{$id}->{$name};
}

sub set {
  my $c    = shift;
  my $id   = shift;
  my $name = shift;
  my $val  = shift;

  die "Not enough params" unless ($id && $name);

  $current->{stash}->{$id}->{$name} = $val;
  return $val;
}

sub option {
  my $s = shift;
  my $option = shift;
  my $val = shift;

  if (defined $val) {
	my $old_val = $current->{options}->{$option};
	$current->{options}->{$option} = $val;
	$s->notify($option.'_changed', old_value=>$old_val, new_value=>$val )
		if ($old_val ne $val);
  }
  return $current->{options}->{$option};
}

sub get_module_option {
        my $s = shift;
        my %a = @_;
        return $current->{$a{type}}->{$a{id}}->{$a{option}};
}

sub set_module_option {
        my $s = shift;
        my %a = @_;
        $current->{$a{type}}->{$a{id}}->{$a{option}} = $a{value};
        return 1;
}

sub load {
   my $c = shift;
   $current = $defaults;
   unless ((-d "$ENV{HOME}/.syndigator") && ((-f "$ENV{HOME}/.syndigator/$PREFS_FILE")) ) {
	return 1;
   }

   if ( $ENV{SYNDIGATOR_DEBUG} ) {
     unless ( open(PREF, "<$ENV{HOME}/.syndigator/preferences.xml") ) {
	 $@ = "Could not create the preferences file: $!";
         return undef;
     }

     $/ = undef;
     my $xml = <PREF>;

     if (! $xml ) {
   	close(PREF);
	return 1;
     }

     eval {
       my $xs = new XML::Simple(NoAttr => 1, KeepRoot => 1);
       $loaded = $xs->XMLin($xml)->{opt};
       foreach my $k (keys %$loaded) {
	if (ref($loaded->{$k}) eq 'HASH') {
	  foreach my $k2 (keys %{$loaded->{$k}}) {
		$current->{$k}->{$k2} = $loaded->{$k}->{$k2};
	  }
	} else {
	  $current->{$k} = $loaded->{$k};
	}
       }
     };
     if ($@) {
	$@ = "Could not load preferences";
   	close(PREF);
	return undef;
     }

     close(PREF);

   } else {
	$current = retrieve("$ENV{HOME}/.syndigator/$PREFS_FILE");
   } 

   return 1;

}

sub save {
   my $s = shift;
   $s->log_spam( Dumper($current) );
   if ( ! -d "$ENV{HOME}/.syndigator" ) {
	eval {
	   mkpath("$ENV{HOME}/.syndigator",0,0700);
 	};
	if ($@) {
	  $@ = "Could not create the syndigator save dir";
	  return undef;
	}
   }

   if ($ENV{SYNDIGATOR_DEBUG}) {
     my $xml;
     eval {
  	my $xs = new XML::Simple(NoAttr => 1,KeepRoot => 1);
	$xml = $xs->XMLout($current);
     };   
     if ($@) {
	$@ = "Could not save preferences";
	return undef;
     }

     unless ( open(PREF, ">$ENV{HOME}/.syndigator/$PREFS_FILE") ) {
	$@ = "Could not create the preferences file: $!";
        return undef;
     }

     print PREF $xml;

     close(PREF);

   } else {
	store $current, "$ENV{HOME}/.syndigator/$PREFS_FILE";
   }

   return 1;
}

################################################
# Global/direct variables.
################################################
sub save_on_exit {
        my $s=shift;
        if (@_) {
           $current->{options}->{save_on_exit}=$_[0];
           $s->notify('save_on_exit_changed');
        }
        return $current->{options}->{save_on_exit}
}
sub online {
        my $s=shift;
        if (defined($_[0])) {
           $current->{options}->{online}=$_[0];
           $s->notify('online_changed');
        }
        return $current->{options}->{online}

}
sub per_view_columns {
        my $s=shift;
        if (defined($_[0])) {
           $current->{options}->{per_view_columns}=$_[0];
           $s->notify('per_view_columns_changed');
        }
        return $current->{options}->{per_view_columns}
}
sub per_view_pane {
        my $s=shift;
        if (defined($_[0])) {
           $current->{options}->{per_view_pane}=$_[0];
           $s->notify('per_view_pane_changed');
        }
        return $current->{options}->{per_view_pane}
}

####################################################

our $current = { };

our $defaults = { 
	options => { 
			per_view_columns 	=> 0, 
			per_view_pane		=> 0,
			fetch_interval     	=> 60,
                        auto_save          	=> 1,
                        auto_save_interval 	=> 30,
                        save_on_exit       	=> 1,
                        online             	=> 1,
			show_category_col	=> 1,
			colored_diffs		=> 1,
		   },
	
};

1;
